extensions [ time  profiler pathdir ]

globals
[
  ERROR-CONDITION?
  startdate
  stopdate
  day-before-stopdate ;; to make sure that the simulation stops also in cases when no infected people remain
  weekend?            ;; tells whether the current day is a Saturday or a Sunday
  past-events-counter ;; only for debugging purposes
  NULL                ;; a name for the empty string
;; these globals whose names end in -name are for outputting result files
  logfile-name
  resultfile-name
  profile-name
  worldfile-name
  directory-name
;;
  period             ;; contains information whether there is a lockdown or not
  results            ;; certain results county-wise
  log-level          ;; decides what is to be written to ./log.txt
  colour-names       ;; for the colours of the counties
  day-names          ;; for the startdate and the stopdate monitor and for logging information
  person-zero        ;; the first infectious person
;; these globals whose names begin with newly- are for calculating moving averages and for calculating the
;; the basic reproduction number estimates
  newly-deceased
  newly-deceased-7-days
  newly-deceased-7-days-ma
  newly-deceased-7-days-ma-max
  newly-deceased-reported
  newly-infected
  newly-infected-reported
  newly-infected-7-days
  newly-infected-7-days-ma
  newly-infected-7-days-ma-max
  newly-infected-4-days
  newly-infected-4-days-ma
  newly-infected-7-days-ma-5-days
  newly-infected-4-days-ma-5-days
;; added 2010-10-24 to avoid calculating these values within the R0 plot
  reproduction-number            ;; basic reproduction number = mean of the numbers all current infectious agents have infected
  var-infected                   ;; variance of the numbers all current infectious agents have infected
  dispersion-parameter           ;; parameter of overdispersion
;; added 2010-10-25 to avoid calculating this agentset over and over again
  infectious-people
;;
  total-infectiosity-rate        ;; the current rate of infectious persons among all persons
  pandemic-grows?                ;; signals whether the pandemic is growing
  all-ever-infected              ;; the number of all persons which were ever infected
;; the following globals whose names begin with max- are only used for rescaling the histogram plots
  max-freq-ccs      ;; the frequency of the mode of the distribution of the product of conviviality and carelessness of susceptible agents
  max-freq-cci      ;; the frequency of the mode of the distribution of the product of conviviality and carelessness of infected agents
  max-freq-ccp      ;; the frequency of the mode of the distribution of the product of conviviality, carelessness and individually perceived risk of all agents
  max-freq-pir      ;; the frequency of the mode of the distribution of the individually perceived risk of all agents
  max-freq-ns       ;; tghe meaximum of the frequencies of the distributions of the norm saliences
;;
  mean-distance-to-nearest-neighbour  ;; the mean of the distance to each agent's nearest neighbour
  end-of-run?                ;; is true when the last infectious agent is removed, when this is true all outstanding events such that the model can stop
  multi-run?                 ;; is true when the model is running headless to avoid some huge output
  no-more-infectious-at      ;; the time when the last infectious agent is removed
  this-randomseed            ;; the random seed used in the current run, to be able to re-run the current parameter combination
  random-positive-test-rate  ;; the positive-test rate for a random sample of five per cent of the population
  suspect-positive-test-rate ;; the positibe-test rate for a non-random sample of all neighbours of all infectious agents within a distance that the sample size if five percent
  max-positive-test-rate     ;; the maximum of either test rate over time (to rescale the respective plot)
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; normative framework
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
  all-norms      ;; the names of the norms
  all-norm-types ;; as norms can be grouped to the types afraid of losing health and afraid of losing freedom, these are the types of the norms
  all-actions    ;; the names of the possible actions
;; the globals whose names begin with W are the coefficients of the formula yielding the values of the norm saliences
  WC
  WO
  WNPV
  WP
  WS
  WE
;;
  IDW              ;; the weight of the individual drive as opposed to the normative drive NDW; IDW = 1 - IDW (NDW is set via the user interface)
  WITNESSES        ;; a few agents whose actions are sent to log.txt
  LISTENERS        ;; the number of neighbours which receive norm invocations
  DISCOUNT         ;; a factor by which norm invocations in the agents' memories are discounted
  invocation-count ;; a list for bookkeeping invocations
]

breed [ people human ]
breed [ counties county ]
breed [ border border-post ]

people-own
[
  my-county                          ;; the county the agent usually lives in
  color-of-my-county                 ;; the colour of this county
  borderliner?                       ;; if true, person lives near the border of a county, only used for spreading people over the world and making the border areas less dense
  my-workplace                       ;; an alternative patch where the agent could be found, actually not used
  my-home                            ;; the patch the agents starts in and returns to every weekend
  conviviality                       ;; a constant feature determining the risk the agent takes
  carelessness                       ;; another such feature
  perceived-risk                     ;; the actually perceived risk, derived either from the percentage of agents in the neighbourhood or of the whole territory
  state                              ;; one of susceptible infected ill recovered dead
  state-history                      ;; a complex list keeping information when and where the state changed
  infectious?                        ;; an additional state which can be combined with some of the states mentioned above
  positively-tested?                 ;; another additional state mainly used to avoid re-testing
  whom-I-infected                    ;; an agentset keeping all agents infected by this agent
  infection-chain                    ;; a list containing the information about all agents whose infectiosity led to this agent's infection
  time-of-infection                  ;; time when this agent was infected
  time-of-outbreak                   ;; time when this agent became symptomatic
  time-of-recovery                   ;; time when this agent recovered
  future-moves                       ;; a list of scheduled move events, used to avoid future moves before the next move has happened
  immunity-loss-scheduled?           ;; to avoid double scheduling
  witness?                           ;; whether this agent writes its actions to ./log.txt
  my-risk                            ;; the agent's "objective" risk
  my-neighbours                      ;; the agent's current neighbours
  distance-to-my-potential-infector  ;; used to determine which agent will be the one which infects this agent
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; normative framework
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
  my-norms                        ;; just a copy of all-norms (in fact not necessary in this context, as there is only one breed of agents using norms for decision making)
  my-norm-saliences               ;; a list keeping the agent's current values of the salience of a norm
  my-compliances                  ;; a list keeping the number how often this agent complied with each of the norms
  my-violations                   ;; a list keeping the number how often this agents violated each of the norms
  other-compliances               ;; a list keeping the number how often other agents in the neighbourhood complied with each of the norms
  other-violations                ;; a list keeping the number how often other agents in the neighbourhood violated each of the norms
  punishments                     ;; a list keeping the number how often other agents in the neighbourhood were punished because of violating each of the norms
  sanctions                       ;; a list keeping the number how often other agents in the neighbourhood were sanctioned because of violating each of the norms
  explicit-compliance-invocations ;; a list keeping the number how often this agent received an explicit invocation for each of the norms (as a praise)
  explicit-violations             ;; a list keeping the number how often this agent received an explicit invocation for each of the norms (as a reprimand
  my-party-propensity             ;; this agent's current propensity to go to a party (or elsewhere when not really necessary)
  my-mask-propensity              ;; this agent's current propensity to wear a mask
  my-unmask-propensity            ;; this agent's current propensity to strip its mask off
  my-stay-propensity              ;; this agent's current propensity to stay at home instead of going elsewhere when not really necessary
  individual-drive                ;; the drive to take an action as the result of consider just objective risks
  normative-drive                 ;; the drive to take an action as the result of normative consideration
  last-message-id                 ;; refers to the last message about a punishment, a sanction, or an explicit invocation to make sure that a message updates the normative board only once
  wears-a-mask?                   ;; shorthand for use in the risk-infection procedure
  stays-at-home?                  ;; shorthand for use in the risk-infection procedure
]

counties-own
[
  area                ;; the number of patches belonging to this county
  population          ;; the number of people whose "home" if in this county
  density             ;; population / area
  county-number       ;; to identify a county
  colour              ;; the colour used to make counties distinguishable in the view
  my-patches          ;; set of all patches belonging to this county
  my-borderline       ;; set of all patches belonging to this county and adjacent to another patch
  my-people           ;; set of all people originating in this county
  my-newly-infected   ;; counts the newly infected agents during the current day
  my-newly-deceased   ;; counts the deceased agents of the current day
]

patches-own
[
  my-center   ;; used to identify the county to which this patch belongs (necessary to identify patches at the boundary to other counties)
]

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; setup procedures
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

to setup
  clear-all
;; initialising global variables
  set end-of-run? false
  set NULL ""
  set log-level 0
  set ERROR-CONDITION? false
  set interruptible? false
  set period "normal time"
  set colour-names ( list "red" "orange" "brown" "yellow" "green" "lime" "turquoise" "cyan" "sky" "blue" "violet" "magenta" "pink" )
  set day-names ( list "Mon" "Tue" "Wen" "Thu" "Fri" "Sat" "Sun" )
  if normative-mode? [ setup-normative-framework ]
  let run-identifier format 4 behaviorspace-run-number
  set multi-run? behaviorspace-run-number > 0
;; setting names for output directories and output files
  set directory-name ( word "run_" run-identifier "_" rectify-date )
  pathdir:create directory-name
  set resultfile-name ( word directory-name "/result.csv" )
  set logfile-name ( word directory-name "/log.txt" )
  set profile-name ( word directory-name "/profile.txt" )
  set worldfile-name ( word directory-name "/world.csv" )
;;
  file-open logfile-name
;; setting and keeping the seed used for this run
  set this-randomseed new-seed
  if constant-seed? [ set this-randomseed 118866472 ]
  random-seed this-randomseed
  util-flog-observer 0 ( word "random seed of this run " run-identifier " was " this-randomseed )
;; initialising global variables used for reporting, calculating moving averages and  plotting
  set infectious-people turtle-set nobody
  set newly-deceased-reported 0
  set newly-deceased 0
  set newly-deceased-7-days []
  set newly-infected-reported 0
  set newly-infected 0
  set newly-infected-7-days []
  set newly-infected-4-days []
  set newly-infected-7-days-ma-5-days []
  set newly-infected-4-days-ma-5-days []
  set newly-infected-7-days-ma-max ( list 0 0 )
  set newly-deceased-7-days-ma-max ( list 0 0 )
  set no-more-infectious-at 0
;; these are for initialising the y-range of several histograms
  set max-freq-ccs 0.1
  set max-freq-cci  0.1
  set max-freq-ccp 0.1
  set max-freq-pir 0.1
  set max-freq-ns 0.1
;; just for reporting how may events have already occurred
  set past-events-counter 0
;; preparing for the start of the simulation
;; this is necessary as the startdate must be fixed before the people can successfully initialise their state-history
  reset-ticks
  time:clear-schedule
  set startdate time:create "2020/01/01 00:00"
  set stopdate time:plus startdate max-months "months"
  set day-before-stopdate time:plus stopdate -25 "hours"
  time:anchor-schedule startdate 1 "hour"
  let day-of-week time:get "dayofweek" T TNOW - 1
  set weekend? day-of-week > 4
;; initialising counties and making counties initialise patches and people
  create-counties initial-counties [ counties-setup ]
  setup-patches
  ask counties ;; counties colour their patches and count them to find out their area
  [
    set my-patches patches with [ shade-of? pcolor [ color ] of myself ]
    set area count my-patches
    set my-newly-infected 0
    set my-newly-deceased 0
  ]
  let area-density-population sort-by [ [ m n ] -> item 0 m < item 0 n ] [ ( list area density population ) ] of counties
  let min-area first first area-density-population
  let max-area first last area-density-population
;; counties determine their population density according to density-mode and make a preliminary calculation of their population size
  let overall-population 0
  ask counties
  [
    let this-county self
    let this-color color + 3
    if density-mode = "all counties equal" [ set density 0.8 ]
    if density-mode = "linearly decreasing" [ set density 0.9 - 0.8 / ( max-area - min-area ) * ( area - min-area ) ]
    if density-mode = "d~1/area" [ set density 1 / ( 80 / 9 * ( area - min-area ) / ( max-area - min-area ) + 10 / 9 ) ]
    set population ( area * density )
    set overall-population overall-population + population
  ]
;; given the population density per county we have a preliminary population size per county
;; to populate the whole world with total-population from the GUI the county populations need be adapted
  ask counties
  [
    set population round ( population * total-population / overall-population )
  ]
;; due to rounding the sum of the population is not equal total-population, check this
  let diff-pop total-population - sum [ population ] of counties
  ifelse diff-pop < 0
  [ while [ diff-pop < 0 ] [ ask one-of counties [ set population population - 1 ] set diff-pop diff-pop + 1 ] ]
  [ while [ diff-pop > 0 ] [ ask one-of counties [ set population population + 1 ] set diff-pop diff-pop - 1 ] ]
;; now the number of the total population is correct, people can be spread over the county territories
;; the borderlines of the counties will be less densely populated than the interior of the counties
  ask counties
  [
    let this-county self
    let this-color color + 3
    let my-occupyable-patches my-patches with [ pcolor mod 10 = 9 ]
    set my-borderline my-patches with [ pcolor mod 10 != 9 ]
    ask my-borderline
    [
      sprout-people 1 [ set my-county this-county setup-people set borderliner? true]
      sprout-border 1 [ set shape "x" hide-turtle set size 1 set color this-color - 6 ]
      set pcolor this-color + 3
    ]
    let people-to-sprout round ( population - count my-borderline )
    ask one-of neighbors [ sprout-people people-to-sprout [ set my-county this-county setup-people set borderliner? false] ]
    set my-people people with [ my-county = myself ]
    set population count my-people
    set density population / area
    ask my-people
    [ if not borderliner?
      [
        move-to one-of my-occupyable-patches
        right random 360 forward 0.4
        set my-home patch-here
      ]
    ]
  ]
  set area-density-population sort-by [ [ m n ] -> item 0 m < item 0 n ] [ ( list area density population ) ] of counties
;; initialising plots
  setup-plots
;; placing and initialising person-zero
  let densest-county max-one-of counties [ density ]
  let sparsest-county min-one-of counties [ density ]
  if zero-person = "densest" [ set person-zero one-of people with [ my-county = densest-county ] ]
  if zero-person = "sparsest" [ set person-zero one-of people with [ my-county = sparsest-county ] ]
  if zero-person = "random" [ set person-zero one-of people ]
  if zero-person = "center" [ set person-zero min-one-of people [ xcor * xcor + ycor * ycor ] ]
  set end-of-run? false
  ask person-zero
  [
    set-state "infected"
    set-infectious
    set past-events-counter past-events-counter - 1
    set newly-infected newly-infected + 1
    ask my-county [ set my-newly-infected my-newly-infected + 1 ]
    set all-ever-infected all-ever-infected + 1
    report-tested
  ]
;; completing the task of initialising people
  set witnesses n-of 2 people
  ask witnesses [ set witness? true ]
  if normative-mode? [ set-normative-predispositions ]
;; recolouring counties
  ask counties [ set color [ 0 0 0 40 ] ]
;; calculating the final population density for adjusting infection distances to the size of the population
  set mean-distance-to-nearest-neighbour distance-to-nearest-neighbour
end

;; calculates the mean distance of all agents to their nearest neighbpur
to-report distance-to-nearest-neighbour
  let result 0
  ask people
  [
    set result result + distance min-one-of other people [ distance myself ]

  ]
  set result result / total-population
  report result
end

;; initialising the global variables for the normative mode of the model
to setup-normative-framework
  set NULL ""
  set all-actions    ( list "PUTTING ON MASK" "GOING TO PARTY" "PUTTING OFF MASK" "STAYING AT HOME" )
  set all-norms      ( list "NORM: WEAR A MASK" "NORM: BREATHE FREELY" "NORM: STAY AT HOME" "NORM: GO TO PARTY" )
  set all-norm-types ( list "FEAR PANDEMIC"     "FEAR OPPRESSION"      "FEAR PANDEMIC"      "FEAR OPPRESSION"   )
;; the weights for calculating the salience of a norm according to ODD, Appendix "Minding Norms in a Nutshell"
  set WC 0.99
  set WO 0.33
  set WNPV 0.66
  set WP 0.33
  set WS 0.99
  set WE 0.99
;; NDW (normative drive weight) is set in the GUI, IDW (individual drive weight) is just 1-NDW
  set IDW 1 - NDW
  set invocation-count ( list ( list "NORM: WEAR A MASK   " ( list 0 0 0 0 0 0 0 0 ) )
                              ( list "NORM: BREATHE FREELY" ( list 0 0 0 0 0 0 0 0 ) )
                              ( list "NORM: STAY AT HOME  " ( list 0 0 0 0 0 0 0 0 ) )
                              ( list "NORM: GO TO PARTY   " ( list 0 0 0 0 0 0 0 0 ) ) )
  set LISTENERS 5
  set DISCOUNT 0.95
end

;; setup procedures for the different agent types follow
to counties-setup
  set county-number who mod initial-counties
  set color item county-number [ 13 23 33 43 53 63 73 83 93 103 113 123 133 ]
  set colour item county-number colour-names
  set shape "Circle 2"
  set size 2
  move-to one-of patches with [ not any? other turtles-here ]
end

to-report different-colour-in-neighbourhood? ;; does what its name says :-)
  let result true
  ask neighbors [ set result result and shade-of? pcolor [ pcolor ] of myself ]
  report not result
end

to setup-patches
  ask patches
  [
    set my-center min-one-of counties [ distance myself ]
    set pcolor 6 + [ color ] of my-center
  ]
  ask patches
  [
    if different-colour-in-neighbourhood? [ set pcolor [ color ] of my-center ]
  ]
end

to setup-people
  set color gray - 2
  set shape "person"
  set size 0.6

;; initialising agents' instance variables
;; in the heterogeneous case the following two variables are beta distributed between 0 and 1 symmetric with respect to 0.5
;; such that mean, mode and median are 0.5
  ifelse heterogeneity = "none" or heterogeneity = "wrt norm" [ set conviviality 0.5 ] [ set conviviality random-beta 2 2 ]
  ifelse heterogeneity = "none" or heterogeneity = "wrt norm" [ set carelessness 0.5 ] [ set carelessness random-beta 2 2 ]
  set future-moves 0
  set immunity-loss-scheduled? false
  set-state "susceptible"
  set infectious? false
  set my-home patch-here
  set color-of-my-county [ color ] of my-county
  set positively-tested? false
  set infection-chain []
  set whom-I-infected []
  set witness? false
  set wears-a-mask? false
  set stays-at-home? false
  set distance-to-my-potential-infector 100 ;; 100 is the maximum possible distance, minimum is determined later
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; initialising the normative framework
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
  if normative-mode?
  [
    set my-norms all-norms
    set my-norm-saliences n-values length my-norms [ 0.0 ]
    set my-compliances n-values length my-norms [ 0 ]
    set my-violations n-values length my-norms [ 0 ]
    set my-party-propensity 0
    set my-mask-propensity 0
    set my-unmask-propensity 0
    set my-stay-propensity 0
    set other-compliances n-values length my-norms [ 0 ]
    set other-violations n-values length my-norms [ 0 ]
    set punishments n-values length my-norms [ 0 ]
    set sanctions n-values length my-norms [ 0 ]
    set explicit-compliance-invocations n-values length my-norms [ 0 ]
    set explicit-violations n-values length my-norms [ 0 ]
  ]
end

;; filling the agents' memories with norm relevant information from the time before the simulation began
to set-normative-predispositions
;; BACKGROUND is the mean of initial invocations of fear pandemic norms, initial compliance dispositions (LNP) of these norms are
;; beta distributed with mean BACKGROUND and range 20; if LNP turns out to be negativem it is set to 0
;; and the initial violation disposition (TNP) is set to the absolute value of LNP; for fear oppression norms, the opposite applies
  ask people
  [
    let my-norm-compliance-dispositions []
    let my-norm-violation-dispositions []
    foreach my-norms
    [ norm ->
      let individual-LNP 0
      ifelse heterogeneity = "none" or heterogeneity = "wrt risk"
      [ set individual-LNP BACKGROUND ]
      [ set individual-LNP round ( BACKGROUND + ( random-beta 2 2 - 0.5 ) * 20 ) ]
      let individual-TNP 0
      if individual-LNP < 0
      [
        set individual-TNP 0 - individual-LNP
        set individual-LNP 0
      ]
      util-flog 6 ( word "Norm: " norm ", BACKGROUND: " BACKGROUND ", individual-LNP: " individual-LNP ", individual-TNP: " individual-TNP )
      let np position norm all-norms
      if item np all-norm-types = "FEAR PANDEMIC"
      [
        set my-norm-compliance-dispositions lput individual-LNP my-norm-compliance-dispositions
        set my-norm-violation-dispositions lput individual-TNP my-norm-violation-dispositions
      ]
      if item np all-norm-types = "FEAR OPPRESSION"
      [
        set my-norm-compliance-dispositions lput individual-TNP my-norm-compliance-dispositions
        set my-norm-violation-dispositions lput individual-LNP my-norm-violation-dispositions
      ]
    ]
    set my-compliances my-norm-compliance-dispositions
    set my-violations my-norm-violation-dispositions
    set other-compliances my-norm-compliance-dispositions
    set other-violations my-norm-violation-dispositions
    set punishments  my-norm-violation-dispositions
    set sanctions  my-norm-violation-dispositions
    set explicit-compliance-invocations my-norm-compliance-dispositions
    set explicit-violations  my-norm-violation-dispositions
    if witness? [ util-flog 7 ( word "initial disposition comply: " my-norm-compliance-dispositions " violate: " my-norm-violation-dispositions ) ]
  ]
end

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; procedures to start the simulation and keep it running
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
to go
  set interruptible? false
  set end-of-run? false
  if not multi-run?  ;; if NetLogo runs in behaviour space the profiler is switched off
  [
    profiler:start
    profiler:reset
    profiler:start
  ]
  prepare-results
  time-schedule-days ( time:plus startdate 1 "minutes" ) ;; the next daily statistic is scheduled
  util-flog-observer 2 ( word "model started at " T TNOW " the calendar length is " time:size-of-schedule ", real time is " rectify-date )
  ask people ;; all agents define their first moving event
  [
    time-schedule-event self [ [] -> move-around ] "move around" ( time:plus T TNOW random-float 1.0 "days"  )
  ]
;; for the variant with infections later during the run, these events are scheduled
  if infectious-from-abroad = "months 2..11"
  [
    ask n-of ( total-population / 50 ) people
    [ time-schedule-event self [ [] -> return-from-abroad ] "return-from-abroad" ( time:plus T TNOW ( 31 + random-float 304 ) "days" ) ]
  ]
  if infectious-from-abroad = "months 7..8"
  [
    ask n-of ( total-population / 50 ) people
    [ time-schedule-event self [ [] -> return-from-abroad ] "return-from-abroad" ( time:plus T TNOW ( 182 + random-float 62 ) "days" ) ]
  ]
  if infectious-from-abroad = "months 9..11"
  [
    ask n-of ( total-population / 50 ) people
    [ time-schedule-event self [ [] -> return-from-abroad ] "return-from-abroad" ( time:plus T TNOW ( 244 + random-float 91 ) "days" ) ]
  ]
  util-flog-observer 2 ( word "people activated for the first time at " T TNOW " the calendar length is " time:size-of-schedule )
;; more events will be scheduled in the course of the simulation, the simulation ends at stopdate, but if interrupted or no infectious agents left
;; scheduled events are deleted, and stopdate is reached
  if time:size-of-schedule > 0
  [
    time:go-until stopdate
  ]
  report-final-results
end


;; a utility which moves an agent to an allowable place depending on the kind of period (lockdown ornot) and its own state as far-commuter
to distance-as-regulated
  if period = "normal time"
  [
    ifelse random-float 1.0 < far-commuter
    [ forward long-distance util-flog 50 ( word "moved " long-distance " " ifelse-value infectious? [ "" ] [ "not " ] "infectious." ) ]
    [ forward short-distance ]
  ]
  if period = "lockdown"
  [
    ifelse random-float 1.0 < far-commuter / 2
    [ forward short-distance util-flog 50 ( word "moved " ( long-distance / 5 ) " " ifelse-value infectious? [ "" ] [ "not " ] "infectious." ) ]
    [ forward 0.1 ]
  ]
;; quarantine is currently not implemented
  if period = "quarantine" [ if any? people in-radius 5  with [ infectious? ] [ forward 0 ] ]
end

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; agents' procedures to move around and change their infection state
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

to move-around ;; general procedure to branch to one of the special move procedures
  set future-moves future-moves - 1                    ;; this line to prevent an agent from scheduling more than one move in the future
  set past-events-counter past-events-counter + 1
  if state != "ill" and state != "dead"
  [
     if normative-mode? [ update-normative-board ]
     move-all-mobile
  ]
  let next-move ( time:plus T TNOW ( 1 + random-float 2 ) "days"  )

  if state = "susceptible" [ move-susceptible next-move ]
  if state = "infected" [ move-infected ]
  if state = "ill" [ move-ill ]
  if state = "recovered" [ move-recovered next-move ]
  if state = "dead" [ move-dead ]
  if state = "immune" [ move-immune next-move ]
  if infectious? [ set shape "default" ]
end

to move-all-mobile  ;; common procedure for all which are mobile (not ill and not dead), also prevents agents to move beyond the boundaries of the world
  ifelse abs(xcor) > 49 or abs (ycor) > 49 or weekend?
  [ go-home]
  [
    right random 360
    distance-as-regulated
    set state-history lput ( list state T TNOW patch-here ) state-history
  ]
end

to move-susceptible [ next-move ] ;; procedure for susceptible agents

  ifelse any? people in-radius ( distance-for-infection * mean-distance-to-nearest-neighbour ) with [ infectious? ] and random-float 1.0 < infections-per-contact
  [
    risk-infection
  ]
  [
    ifelse future-moves < 1
    [ time-schedule-event self [ [] -> move-around ] "move around" next-move ]
    [ util-flog 20 ( word "future-moves = " future-moves ", not scheduled." ) ]
  ]
end

to move-infected ;; procedure for infected agents to fall ill or recoves asymptomatically
  ifelse random-float 1.0 < asymptomatic-per-infected
  [ time-schedule-event self [ [] -> recover-or-die ] "recover-or-die" ( time:plus T TNOW illness-duration "days" ) ]
  [ time-schedule-event self [ [] -> fall-ill ] "fall ill" ( time:plus T TNOW incubation-time "days" ) ]
  if random-float 1.0 < tested-per-infected and not positively-tested?
  [
    set positively-tested? true
    if random-float 1.0 < reporting-rate [ report-tested ]
    go-home
  ]
end

to move-ill ;; procedure for sick agents to recover or die
  time-schedule-event self [ [] -> recover-or-die ] "recover-or-die" ( time:plus T TNOW illness-duration "days" )
  if not positively-tested?
  [
    set positively-tested? true
    if random-float 1.0 < reporting-rate [ report-tested ]
  ]
end

to move-recovered [ next-move ] ;; procedure for recovered agents to continue moving at time=next-move
  if future-moves < 1 [ time-schedule-event self [ [] -> move-around ] "move around" next-move ]
end

to move-dead ;; proedure to reset some instance variable after the end of the agent's life
  set positively-tested? false
  set infectious? false
  set shape "Cross"
  set color black
end

to move-immune [ next-move ] ;; procedure to have immune agents lose their immunity
  let immunity-duration ( time:plus T TNOW ( random-float-in-range immunity-duration-min immunity-duration-max ) "months" )
  if future-moves < 1 [ time-schedule-event self [ [] -> move-around ] "move around" next-move ]
  if not immunity-loss-scheduled? [ time-schedule-event self [ [] -> lose-immunity ] "lose-immunity" immunity-duration ]
  set immunity-loss-scheduled? true
end




;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; health related procedures
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
to lose-immunity  ;; what happens if an agent loses immunity
  set past-events-counter past-events-counter + 1
  set-state "susceptible"
  if future-moves < 1
  [
    let next-move ( time:plus T TNOW ( 1 + random-float 2 ) "days"  )
    time-schedule-event self [ [] -> move-around ] "move around" next-move
  ]
end

to risk-infection  ;; taking the risk of being infected, dependent on own precautions and distance to nearest infectious agent
;; the infection risk depends on how convivial and how careless an agent is and the general risk to meet an infectious person
  let risk-to-be-infected conviviality * carelessness * ( 1 - perceived-risk )
;; and is reduced by wearing a mask or staying at home
  if wears-a-mask? [ set risk-to-be-infected risk-to-be-infected * 0.2 ]
  if stays-at-home? [ set risk-to-be-infected 0.0 ]
  ifelse risk-to-be-infected > random-float 1.0
  [ ;; the risk has become manifest
    util-flog 20 ( word "risk " precision risk-to-be-infected 4 " " precision ( conviviality * carelessness ) 4 " infected" )
    let my-potential-infector min-one-of ( infectious-people ) [ distance self ]
    set distance-to-my-potential-infector distance my-potential-infector
    util-flog 2 ( word "my potential infector is " my-potential-infector ", " distance-to-my-potential-infector " patches away." )
    let my-infector one-of people in-radius ( distance-for-infection * mean-distance-to-nearest-neighbour ) with [ infectious? ]
    set infection-chain fput ( list my-infector T TNOW patch-here ) [ infection-chain ] of my-infector
    ask my-infector [ set whom-I-infected lput myself whom-I-infected ]
    set-state "infected"
    set newly-infected newly-infected + 1
    set all-ever-infected all-ever-infected + 1
    let next-move ( time:plus T TNOW ( 12 + random-float 2.0 ) "days"  )
    let time-to-infectiosity ( time:plus T TNOW ( random-float-in-range time-to-infectiosity-min time-to-infectiosity-max ) "days" )
    util-flog 5 ( word "is now infected and will move again at " next-move )
    time-schedule-event self [ [] -> set-infectious ] "set-infectious" time-to-infectiosity
    if future-moves < 1 [ time-schedule-event self [ [] -> move-around ] "move around" next-move ]
    if random-float 1.0 < tested-per-infected and not positively-tested?
    [
      set positively-tested? true
      if random-float 1.0 < reporting-rate [ report-tested ]
    ]
  ]
  [ ;; tghe risk has not realised
    util-flog 20 ( word "risk " precision risk-to-be-infected 4 " " precision ( conviviality * carelessness ) 4 " not infected" )
    set state-history lput ( list "not infected" T TNOW patch-here ) state-history
    if future-moves < 1
    [
      let next-move ( time:plus T TNOW ( 1 + random-float 2 ) "days"  )
      time-schedule-event self [ [] -> move-around ] "move around" next-move
    ]
  ]
end

to set-infectious   ;; collects all measures taken when a person is infectious in the course of its own being infected
  set past-events-counter past-events-counter + 1
  if state = "dead" or state = "immune" or infectious? [ stop ] ;; lines like this are necessary as future events are unforeseeable :-)
  set infectious? true  set shape "default"
  set size 2
  set state-history lput ( list "infectious" T TNOW patch-here ) state-history
end

to return-from-abroad  ;; collects all measures taken when a person is infectious without being infected within the simulation
  set past-events-counter past-events-counter + 1
  if state = "dead" or state = "immune" or infectious? [ stop ]
  set-state "infected"
  set infectious? true  set shape "default"
  set size 2
  set state-history lput ( list "infectious from abroad" T TNOW patch-here ) state-history
  output-show ( word T TNOW " infected from abroad" )
end

to fall-ill    ;; collects all measures taken when an agent falls ill
  set past-events-counter past-events-counter + 1
  ifelse state = "ill"
  [
    util-flog 15 ( word "fall-ill cannot be entered by people with " state )
  ]
  [
    let my-hospital one-of [ neighbors ] of my-county
    setxy [ pxcor ] of my-hospital [ pycor ] of my-hospital
    set-state "ill"
    time-schedule-event self [ [] -> recover-or-die ] "recover-or-die" ( time:plus T TNOW illness-duration "days" )
    if not positively-tested?
    [
      set positively-tested? true
      report-tested
    ]
  ]
end


to go-home  ;; return to original patch every weekend, after returning from hospital or when dead
  setxy [ pxcor ] of my-home [ pycor ] of my-home
end

to become-immune  ;; collects all changes to be made when an agents becomes immune
  set past-events-counter past-events-counter + 1
  set-state "immune"
  set infectious? false
  set shape state-shape
  set size 0.6
  set positively-tested? false
  set whom-I-infected []
end

to recover-or-die   ;; collects all state changes when an infected agents recovers or dies (is removed, in the SIR terminology)
  set past-events-counter past-events-counter + 1
  ifelse state = "recovered" or state = "dead"
  [
    util-flog 20 ( word "recover-or-die cannot be entered by people with " state )
  ]
  [
    ifelse random-float 1.0 < case-fatality-rate and state = "ill"
    [
      go-home
      set-state "dead"
      set infectious? false
      set shape "Cross"
      set size 2
      set newly-deceased newly-deceased + 1
      ask my-county [ set my-newly-deceased my-newly-deceased + 1 ]
      report-death
    ]
    [
      go-home
      set-state "recovered"
      let time-till-immunity random-float-in-range time-till-immunity-min time-till-immunity-max
      if future-moves < 1 [ time-schedule-event self [ [] -> move-around ] "move around" ( time:plus T TNOW 4 "days" ) ]
      time-schedule-event self [ [] -> become-immune ] "become immune" ( time:plus T TNOW time-till-immunity "days" )
    ]
  ]
end

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; reporting procedures
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
to report-final-results
  file-close-all
  file-open resultfile-name                      ;; outputs county related statistics
  foreach n-values initial-counties [ n -> n ] [ list-item ->        ;; the headline
    file-type "time, county,"
    file-type ( word "area" format 2 list-item ",")
    file-type ( word "density" format 2 list-item "," )
    file-type ( word " population" format 2 list-item ", " )
    file-type ( word " susceptible" format 2 list-item ", " )
    file-type ( word " infected" format 2 list-item ", " )
    file-type ( word " ill" format 2 list-item ", " )
    file-type ( word " recovered" format 2 list-item ", " )
    file-type ( word " immune" format 2 list-item ", " )
    file-type ( word " dead" format 2 list-item ", " )
    file-type ( word " infectious" format 2 list-item ", " )
    file-type ( word " newly-infected" format 2 list-item ", " )
    file-type ( word " newly-deceased" format 2 list-item ", " )
    file-type ( word " R0" format 2 list-item ", " )
  ]
  file-print ""
  foreach results                            ;; the final statistics for each county
  [ ?1 ->
    let daily-result ?1
    ( foreach n-values initial-counties [ ??1 -> ??1 ] daily-result
    [ [??1 ??2] ->
      let county-result ??2
      file-type ( word item 0 county-result"," ) ;; time
      file-type ( word item 1 county-result"," ) ;; county
      file-type ( word item 4 county-result"," ) ;; area
      file-type ( word item 5 county-result"," ) ;; density
      file-type ( word item 6 county-result"," ) ;; population
      file-type ( word item 7 county-result"," ) ;; susceptible
      file-type ( word item 8 county-result"," ) ;; infected
      file-type ( word item 9 county-result"," ) ;; ill
      file-type ( word item 10 county-result"," ) ;; recovered
      file-type ( word item 11 county-result"," ) ;; immune
      file-type ( word item 12 county-result"," ) ;; dead
      file-type ( word item 13 county-result"," ) ;; infectious
      file-type ( word item 14 county-result"," ) ;; newly infected
      file-type ( word item 15 county-result"," ) ;; newly deceased
      file-type ( word item 16 county-result"," ) ;; R0
    ]
    )
    file-print ""
  ]
  file-flush
  file-close-all
;; exporting world (if not headless) and plots
  if not multi-run? [ export-world worldfile-name ]
  export-plot "Current state of the population 1" ( word directory-name "/history1.csv" )
  export-plot "Current state of the population 2" ( word directory-name "/history2.csv" )
  export-plot "Infections in last 24 hours" ( word directory-name "/infections.csv" )
  export-plot "Deaths in last 24 hours" ( word directory-name "/deaths.csv" )
  export-plot "R0 real / est" ( word directory-name "/R0.csv" )
  export-plot "whom-I-infected" ( word directory-name "/whom-I-infected.csv" )
  export-plot "whom-I-infected_t" ( word directory-name "/whom-I-infected-t.csv" )
  export-plot "norm actions" ( word directory-name "/norm actions.csv" )
  export-plot "med perc risk" ( word directory-name "/perc risk.csv" )
  export-plot "sal WEAR A MASK" ( word directory-name "/sal WEAR A MASK.csv" )
  export-plot "sal BREATHE FREELY" ( word directory-name "/sal BREATHE FREELY.csv" )
  export-plot "sal STAY AT HOME" ( word directory-name "/sal STAY AT HOME.csv" )
  export-plot "sal GO TO PARTY" ( word directory-name "/sal GO TO PARTY.csv" )
  export-plot "test positivity rate" ( word directory-name "/test positivity rate.csv" )
  file-close-all
;; exporting the profiler results (if not headless)
  if not multi-run?
  [
    profiler:stop
    file-open profile-name
    file-print profiler:report
    file-close-all
    profiler:reset
  ]
  file-open logfile-name
;; writing the final state of output metrics to ./log.txt
  util-flog-observer -2 ( word "model stopped at " T TNOW " the calendar length is " time:size-of-schedule ", real time is " rectify-date )
  util-flog-observer -2 ( word "normative-mode   : " normative-mode? )
  util-flog-observer -2 ( word "BACKGROUND       : " BACKGROUND )
  util-flog-observer -2 ( word "density-mode     : " density-mode )
  util-flog-observer -2 ( word "heterogeneity    : " heterogeneity )
  util-flog-observer -2 ( word "this-randomseed  : " this-randomseed )
  util-flog-observer -2 ( word "still susceptible: " count people with [ state = "susceptible" ] )
  util-flog-observer -2 ( word "infected         : " count people with [ state = "infected" ] )
  util-flog-observer -2 ( word "ill              : " count people with [ state = "ill" ] )
  util-flog-observer -2 ( word "recovered        : " count people with [ state = "recovered" ] )
  util-flog-observer -2 ( word "immune           : " count people with [ state = "immune" ] )
  util-flog-observer -2 ( word "dead             : " count people with [ state = "dead" ] )
  util-flog-observer -2 ( word "infectious       : " count people with [ state = "infectious" ] )
  util-flog-observer -2 ( word "median co ca su  : " median [ conviviality * carelessness ] of people with [ state = "susceptible" ] )
  util-flog-observer -2 ( word "median co ca su  : " median [ conviviality * carelessness ] of people with [ state != "susceptible" ] )
  util-flog-observer -2 ( word "wear a mask      : " count people with [ wears-a-mask? ] )
  util-flog-observer -2 ( word "stay at home     : " count people with [ stays-at-home? ] )
  util-flog-observer -2 ( word "max new infected ; " newly-infected-7-days-ma-max )
  util-flog-observer -2 ( word "max new deaths   : " newly-deceased-7-days-ma-max )
  util-flog-observer -2 ( word "no more infectious at: " no-more-infectious-at )
end

;; collecting county statistics
to prepare-results
  if results = 0 [ set results [] ]
  let this-row []
  foreach n-values initial-counties [ n -> n ]
  [ county-nr ->
    ask county county-nr
    [
      let my-R0 mean [ length whom-I-infected ] of my-people
      set this-row lput ( list T TNOW who color colour area density population
                          count my-people with [ state = "susceptible" ]
                          count my-people with [ state = "infected" ]
                          count my-people with [ state = "ill" ]
                          count my-people with [ state = "recovered" ]
                          count my-people with [ state = "immune" ]
                          count my-people with [ state = "dead" ]
                          count my-people with [ state = infectious? ]
                          my-newly-infected
                          my-newly-deceased
                          my-R0
                        ) this-row
    ]
  ]
  set results lput this-row results
end



;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; some shorthand procedures
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; just shorthand what has to be done when an agent is tested positive
to report-a-positive-test
  set past-events-counter past-events-counter + 1
  set newly-infected-reported newly-infected-reported + 1
end

;; just shorthand what has to be done when an agent dies
to report-a-death
  set past-events-counter past-events-counter + 1
  set newly-deceased-reported newly-deceased-reported + 1
end

;; just shorthand for a move of an infectious to some distant patch, only used in user interactions
to move-an-infectious-far-away
  if any? infectious-people [ ask one-of infectious-people [ setxy random-xcor random-ycor ] ]
end

;; simulating the delayed report because of weekend,evening and night, used in the next two preocedures
to-report time-of-report
  let delay random-float-in-range 0.5 2.0
  let result ( time:plus T TNOW delay "hours" )
  let out-of-office-delays? random-float 1.0 < reporting-delay
  if out-of-office-delays?
  [
    let workday? false
    let office-hours? false
    let day-of-week time:get "dayofweek" T TNOW - 1
    if day-of-week < 5 [ set workday? true ]
    let time-of-day time:get "hour" T TNOW
    if time-of-day > 7 and time-of-day < 18 [ set office-hours? true ]
    if workday? and office-hours? [ set result ( time:plus  T TNOW ( random-float 2.0 ) "hours" ) ]
    if day-of-week < 4 and not office-hours?
    [
      ifelse time-of-day < 8
      [ set delay 9 - time-of-day ]
      [ set delay 31 - time-of-day ] set result ( time:plus T TNOW delay "hours" )
    ]
    if day-of-week = 4 and not office-hours?
    [ ifelse time-of-day < 8 [ set delay 9 - time-of-day ]
    [ set delay 48 + 31 - time-of-day ] set result ( time:plus T TNOW delay "hours" ) ]
    if day-of-week = 5 [ set delay 57 - time-of-day set result ( time:plus T TNOW delay "hours" ) ]
    if day-of-week = 6 [ set delay 33 - time-of-day set result ( time:plus T TNOW delay "hours" ) ]
  ]
  report result
end

;; report the result of an infectiosity test with a delay defined in time-of-report
to report-tested
  time-schedule-event self [ [] -> report-a-positive-test ] "report a positive test" time-of-report
  util-flog 10 ( word "was tested as infected by " infection-chain " at " item ( time:get "dayofweek" T TNOW - 1 ) day-names
    " " T TNOW " and will be reported to statistics at " T time-of-report )
end

;; report a death with a delay defined in time-of-report
to report-death
  time-schedule-event self [ [] -> report-a-death ] "report a death" time-of-report
  util-flog 15 ( word "passed away as infected by " infection-chain " and will be reported to statistics at " T time-of-report )
end

;; collecting all measures taken when a state changes
to set-state [ a-state ]
  ifelse state = "dead" and a-state != "dead"
  [
    util-flog 15 ( word "is already dead, change to " a-state " is impossible " )
  ]
  [
    ifelse state = a-state
    [
      util-flog 15 ( word "is already " state ", state cannot be changed to " a-state )
    ]
    [
      if a-state = "susceptible" [ set color gray set shape "face happy" set infectious? false set size 0.6 ]
      if a-state = "infected"
      [
        set color red
        set shape "face neutral"
      ]
      if a-state = "ill" [set color violet set shape "face sad" set-infectious set past-events-counter past-events-counter - 1 ]
      if a-state = "recovered" [ set color blue set shape "face happy" set size 0.6 ]
      if a-state = "immune" [ set color green set shape "face happy" set infectious? false set size 0.6 ]
      if a-state = "dead" [ set color black set shape "Cross" set infectious? false set size 2 ]
      set state a-state
      if state-history = 0 [ set state-history [] ]
      set state-history lput ( list a-state T TNOW patch-here ) state-history
    ]
  ]
  ifelse infectious? [ set shape "default" ] [ set shape state-shape ]
end

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; procedures called at the beginning of each new simulated day
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

to allow-interruptions      ;; allows a user interaction at the end of each day, enabling users to stop, interrupt or manage the simulation run
  let answer user-one-of "what to do?"
  [
    "continue"
    "go on forever"
    "print profile and continue forever"
    "halt and print profile and result!"
    "spread"
    "normal time"
    "lockdown"
    "inspect"
  ]
  if first answer = "c" [ set interruptible? true ]
  if first answer = "g" [ set interruptible? false user-message "No further intermediate stop" ]
  if first answer = "p" [ file-close-all file-open profile-name file-print profiler:report file-close-all set interruptible? false ]
  if first answer = "h"
  [
    file-close-all
    file-open profile-name
    file-print profiler:report
    file-close-all
    report-final-results
    error "halted intentionally by user"
  ]
  if first answer = "s" [ move-an-infectious-far-away set interruptible? false ]
  if first answer = "n" [ set period "normal time" set interruptible? false output-print ( word T TNOW ": user lockdown ended" ) ]
  if first answer = "l" [ set period "lockdown" set interruptible? false output-print ( word T TNOW ": user lockdown" ) ]
  if first answer = "i" [ inspect one-of infectious-people set interruptible? false ]
end

;; performs a test on two five per cent samples of the total population
to perform-daily-test
  let n-of-tested-persons total-population / 10
  let random-test-persons n-of n-of-tested-persons people
  let suspect-test-persons min-n-of n-of-tested-persons people [ distance-to-my-potential-infector ]
  set random-positive-test-rate ( count random-test-persons with [ infectious? ] ) / n-of-tested-persons
  set suspect-positive-test-rate ( count suspect-test-persons with [ infectious? ] ) / n-of-tested-persons
  if random-positive-test-rate > max-positive-test-rate [ set max-positive-test-rate precision ( ceiling ( random-positive-test-rate * 1000 ) / 1000 ) 3 ]
  if suspect-positive-test-rate > max-positive-test-rate [ set max-positive-test-rate precision ( ceiling ( suspect-positive-test-rate * 1000 ) / 1000 ) 3 ]
end

;; doing some statistics to make the procedure daily-update more easily legible
to do-some-statistics
  if max [ future-moves ] of people > 1 [ set ERROR-CONDITION? true error "max future-moves > 1" ]
  let day-of-week time:get "dayofweek" T TNOW - 1
  set weekend? day-of-week > 4
  if day-of-week = 5
  [ ask people with [ state = "susceptible" or state = "infected" or state = "immune" or state = "recovered" ] [ go-home ] ]
  set past-events-counter past-events-counter + 1
;; the maximum frequency of the distributions of the folowing variables is only calculated once a week as it is quite time consuming
  if day-of-week < 5
  [
    set max-freq-ccs max-freq 20 [ conviviality * carelessness ] of people with [ state = "susceptible" ]
    set max-freq-cci max-freq 20 [ conviviality * carelessness ] of people with [ state != "susceptible" ]
    set max-freq-ccp max-freq 20 [ conviviality * carelessness * ( 1 - perceived-risk ) ] of people
    set max-freq-pir max-freq 20 [ perceived-risk ] of people
    if normative-mode?
    [
      let mode-norm-saliences []
      foreach [ 0 1 2 3 ] [ mns -> set mode-norm-saliences lput ( max-freq 20 [ item mns my-norm-saliences ] of people ) mode-norm-saliences ]
      set max-freq-ns max mode-norm-saliences
    ]
  ]
;; moving averages are not calculated before the desired number of values is reached
  ifelse length newly-deceased-7-days = 7
  [
    set newly-deceased-7-days fput newly-deceased but-last newly-deceased-7-days
    set newly-deceased-7-days-ma mean newly-deceased-7-days
    if newly-deceased-7-days-ma > item 1 newly-deceased-7-days-ma-max [ set newly-deceased-7-days-ma-max ( list ticks newly-deceased-7-days-ma ) ]
  ]
  [
    set newly-deceased-7-days fput newly-deceased newly-deceased-7-days
  ]
  ifelse length newly-infected-7-days = 7
  [
    set newly-infected-7-days fput newly-infected but-last newly-infected-7-days
    set newly-infected-7-days-ma mean newly-infected-7-days
    if newly-infected-7-days-ma > item 1 newly-infected-7-days-ma-max [ set newly-infected-7-days-ma-max ( list ticks newly-infected-7-days-ma ) ]
    ifelse length newly-infected-7-days-ma-5-days = 5
    [
      set newly-infected-7-days-ma-5-days fput newly-infected-7-days-ma but-last newly-infected-7-days-ma-5-days
    ]
    [
      set newly-infected-7-days-ma-5-days fput newly-infected-7-days-ma newly-infected-7-days-ma-5-days
    ]
  ]
  [
    set newly-infected-7-days fput newly-infected newly-infected-7-days
  ]
  ifelse length newly-infected-4-days = 4
  [
    set newly-infected-4-days fput newly-infected but-last newly-infected-4-days
    set newly-infected-4-days-ma mean newly-infected-4-days
    ifelse length newly-infected-4-days-ma-5-days = 5
    [
      set newly-infected-4-days-ma-5-days fput newly-infected-4-days-ma but-last newly-infected-4-days-ma-5-days
    ]
    [
      set newly-infected-4-days-ma-5-days fput newly-infected-4-days-ma newly-infected-4-days-ma-5-days
    ]
  ]
  [
    set newly-infected-4-days fput newly-infected newly-infected-4-days
  ]
;; finds out whether the number of new infections increases or decreases
  ifelse empty? newly-infected-7-days-ma-5-days
  [ set pandemic-grows? false ]
  [ set pandemic-grows? first newly-infected-7-days-ma-5-days > last newly-infected-7-days-ma-5-days ]
  prepare-results
  set total-infectiosity-rate count infectious-people / count people
;; added 2020-10-25 to calculate the dispersion parameter (reproduction-number now is a global variable instead of a value calculated within a plot)
  set reproduction-number ifelse-value ( count infectious-people  > 0 ) [ mean [ length whom-I-infected ] of infectious-people ] [ 0 ]
  ifelse count infectious-people > 2
  [
    set var-infected variance [ length whom-I-infected ] of infectious-people
    set dispersion-parameter ifelse-value reproduction-number = 0 [ 0 ] [ 1 + ( var-infected - reproduction-number ) / sqr reproduction-number ]
  ]
  [ set dispersion-parameter 0 ]
end

;; updates a number of global and individual variables at the beginning of each simulated day
to daily-update
  if auto-lockdown?          ;; start and end an automatically defined lockdown when certain thresholds are passed
  [
    if sum newly-infected-7-days > count people * lockdown-at / 100000 and period = "normal time"
    [ set period "lockdown" output-print ( word T TNOW ": lockdown " newly-infected-reported ) ]
    if sum newly-infected-7-days < count people * lockdown-end-at / 100000 and period = "lockdown"
    [ set period "normal time" output-print ( word T TNOW ": lockdown ended " newly-infected-reported ) ]
  ]
  set infectious-people people with [infectious? ]
  util-flog-observer -2 ( word "A NEW DAY BEGINS " item ( time:get "dayofweek" T TNOW - 1 ) day-names " " T TNOW ", " count infectious-people " people infectious, real time is " rectify-date )
  do-some-statistics
;; line added 2020-10-25 to keep track of this calculation
  ifelse any? infectious-people and var-infected > 0 and var-infected != reproduction-number
  [
    let p 1 - reproduction-number / var-infected
    let r sqr reproduction-number / ( var-infected - reproduction-number )
    let freq-list []
    foreach n-values 10 [ n -> n ]  [ nfl -> set freq-list lput precision ((  count infectious-people with [ length whom-I-infected = nfl ] ) / count infectious-people ) 3 freq-list ]
    util-flog-observer -2 ( word "R0 = " reproduction-number ", variance = " var-infected ", p = " p ", r = " r ", dispersion parameter = " precision dispersion-parameter 3 ", frequencies = " freq-list ", " precision sum freq-list 3 )
  ]
  [ util-flog-observer -2 "no infectious agents left" ]
;; agents calculate their perceived risk of meeting an infectious person and/or the global risk of being infected
  ask people with [ state = "susceptible" ]
  [
    let neighbours people in-radius 5
    let locally-perceived-risk ( count neighbours with [ infectious? ] / count neighbours )
    ifelse pandemic-grows?
    [ set perceived-risk max list locally-perceived-risk total-infectiosity-rate ]
    [ set perceived-risk min list locally-perceived-risk total-infectiosity-rate ]
  ]
  perform-daily-test
  update-plots
;; global variables holding the number of events of the day are reset
  set newly-deceased-reported 0
  set newly-deceased 0
  set newly-infected-reported 0
  set newly-infected 0
  ask counties [ set my-newly-infected 0 set my-newly-deceased 0 ]
;; determining whether the run can be ended when there are no more infectious agents
  set end-of-run? end-of-run? or not any? infectious-people or time:is-equal T TNOW day-before-stopdate or time:is-after T TNOW day-before-stopdate
  time-schedule-days ( time:plus TNOW 1 "days" )
  if end-of-run? and no-more-infectious-at = 0
  [
    set no-more-infectious-at ticks
    util-flog-observer -2 ( word "end of run, no more infectious at " no-more-infectious-at " " T TNOW " (" ticks ")" )
  ]
  if not end-of-run? and not multi-run?  ;; a last opportunity for user interaction
  [
    if interruptible? [ allow-interruptions ]
  ]
  if not multi-run?     ;; except in behaviour space or headless the state of the interface of the day is sent to a file
  [
    export-interface ( word directory-name "/" format 5 ticks ".png" )
  ]
  ifelse end-of-run?    ;; finally if end-of-run? is true, all scheduled events are deleted such that the stop condition in the go procedure is reached
  [
    time:clear-schedule
    util-flog-observer -2 ( word "end-of-run? is " end-of-run? ", current schedule is " time:show-schedule )
  ]
  [   ;; otherwise some information about the state of the run is written to ./log.txt
    util-flog-observer -2 ( word "end-of-run? is " end-of-run? ", after daily update at " T TNOW " the calendar length is " time:size-of-schedule )
  ]
end



;; two shorthands for random times within intervals
to-report incubation-time
  report random-float-in-range incubation-time-min incubation-time-max
end

to-report illness-duration
  report random-float-in-range illness-duration-min illness-duration-max
end


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; procedures for the normative version
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; calculates the propensities to take each of the possible actions,
;; updates the memory of an agent and
;; sends messages to neighbours (asks neighbours to update their respective memory slots)
to update-normative-board
  set my-risk conviviality * carelessness * ( 1 - perceived-risk )
  set my-neighbours people in-radius 5
  if count my-neighbours > LISTENERS [ set my-neighbours min-n-of LISTENERS my-neighbours [ distance self ] ]
  set my-party-propensity propensity-to "GOING TO PARTY"
  set my-mask-propensity propensity-to "PUTTING ON MASK"
  set my-unmask-propensity propensity-to "PUTTING OFF MASK"
  set my-stay-propensity propensity-to "STAYING AT HOME"
  let propensity-to-take-action random-float 1.0
  let i'll-do-nothing propensity-to-take-action > my-unmask-propensity and propensity-to-take-action > my-party-propensity
  let i'll-unmask false
  let i'll-party false
  if not i'll-do-nothing
  [
    set i'll-party my-party-propensity > my-stay-propensity
    set i'll-unmask my-unmask-propensity > my-mask-propensity
    util-flog 12 ( word "PTTA: " precision propensity-to-take-action 4 " compared to MPP = " precision my-party-propensity 4 " and to MUP = " precision my-unmask-propensity 4
     " N:" i'll-do-nothing ", S:" i'll-party ", B:" i'll-unmask ", CCP: " precision my-risk 4
     " " ifelse-value stays-at-home? [ "sits at home" ] [ "went to work or party" ] " " ifelse-value wears-a-mask? [ "wears-a-mask" ] [ "breathes freely" ] )
    if i'll-unmask
    [
      update-compliances "NORM: BREATHE FREELY"
      update-violations "NORM: WEAR A MASK"
      if any? my-neighbours
      [
        ask my-neighbours
        [
          update-observed-compliances "NORM: BREATHE FREELY"
          update-observed-violations "NORM: WEAR A MASK"
        ]
      ]
    ]
    if wears-a-mask? and not i'll-unmask
    [
      update-compliances "NORM: WEAR A MASK"
      update-violations "NORM: BREATHE FREELY"
      if any? my-neighbours
      [
        ask my-neighbours
        [
          update-observed-compliances "NORM: WEAR A MASK"
          update-observed-violations "NORM: BREATHE FREELY"
        ]
      ]
    ]
    if i'll-party
    [
      update-compliances "NORM: GO TO PARTY"
      update-violations "NORM: STAY AT HOME"
      if any? my-neighbours
      [
        ask my-neighbours
        [
          update-observed-compliances "NORM: GO TO PARTY"
          update-observed-violations "NORM: STAY AT HOME"
        ]
      ]
    ]
    if stays-at-home? and not i'll-party
    [
      update-compliances "NORM: STAY AT HOME"
      update-violations "NORM: GO TO PARTY"
      if any? my-neighbours
      [
        ask my-neighbours
        [
          update-observed-compliances "NORM: STAY AT HOME"
          update-observed-violations "NORM: GO TO PARTY"
        ]
      ]
    ]
    set stays-at-home? not i'll-party
    set wears-a-mask? not i'll-unmask
  ]
end

;; calculates the propensity to take a given action
to-report propensity-to [ action ]
  if not member? action all-actions
  [
    util-flog 5 ( word "propensity-to: " action " is not a valid action." )
    set ERROR-CONDITION? true
    report -1
  ]
  if member? self witnesses
  [
    util-flog 6 ( word "calculating propensity to " action )
    util-flog 7 ( word  ": C: " MP my-compliances " V: "  MP my-violations  " OC: "  MP other-compliances " OV: "  MP other-violations
      " P: " MP punishments  " S: "  MP sanctions " EC: "  MP explicit-compliance-invocations " EV: "  MP explicit-violations
      " SAL: " MP my-norm-saliences )
  ]
  calculate-my-individual-drive-to action
  calculate-my-normative-drive-to action
  let a-prop ( IDW * individual-drive + NDW * normative-drive )
  if a-prop > 1 or a-prop < 0
  [
    util-flog (-2) ( word "ERROR: a-prop out of bounds: " a-prop " = " IDW "*" individual-drive " + " NDW "*" normative-drive )
    set ERROR-CONDITION? true
    error "ERROR: a-prop out of bounds: "
  ]
  if member? self witnesses
  [ util-flog 7 ( word  ": ID = " precision individual-drive 4 ", NF = " precision normative-drive 4 ", propensity to " action " = " precision a-prop 4 ) ]
  report a-prop
end

;; calculates the normative drive to take aspecial action, used in the preceding procedure
to calculate-my-normative-drive-to [ action ]
  if not member? action all-actions
  [
    util-flog 0 ( word "calculate-my-normative-drive-to: " action " is not a valid action." )
    set ERROR-CONDITION? true
    stop
  ]
  if action = "GOING TO PARTY"
  [
    calculate-a-salience "NORM: GO TO PARTY"
    calculate-a-salience "NORM: STAY AT HOME"
    let np-dont position "NORM: STAY AT HOME" my-norms
    let np-do position "NORM: GO TO PARTY" my-norms
    let normative-drive-to-go-to-party item np-do my-norm-saliences
    let normative-drive-not-to-go-to-party item np-dont my-norm-saliences
    ifelse normative-drive-to-go-to-party > normative-drive-not-to-go-to-party
    [
      set normative-drive normative-drive-to-go-to-party
    ]
    [
      set normative-drive 1 - normative-drive-not-to-go-to-party
    ]
  ]
  if action = "STAYING AT HOME"
  [
    calculate-a-salience "NORM: STAY AT HOME"
    calculate-a-salience "NORM: GO TO PARTY"
    let np-do position "NORM: STAY AT HOME" my-norms
    let np-dont position "NORM: GO TO PARTY" my-norms
    let normative-drive-to-stay-at-home item np-do my-norm-saliences
    let normative-drive-not-to-stay-at-home item np-dont my-norm-saliences
    ifelse normative-drive-to-stay-at-home > normative-drive-not-to-stay-at-home
    [
      set normative-drive normative-drive-to-stay-at-home
    ]
    [
      set normative-drive 1 - normative-drive-not-to-stay-at-home
    ]
  ]
  if action = "PUTTING ON MASK"
  [
    calculate-a-salience "NORM: WEAR A MASK"
    calculate-a-salience "NORM: BREATHE FREELY"
    let np-do position "NORM: WEAR A MASK" my-norms
    let np-dont position "NORM: BREATHE FREELY" my-norms
    let normative-drive-to-put-on-mask item np-do my-norm-saliences
    let normative-drive-to-put-off-mask item np-dont my-norm-saliences
    ifelse normative-drive-to-put-on-mask > normative-drive-to-put-off-mask
    [
      set normative-drive normative-drive-to-put-on-mask
    ]
    [
      set normative-drive 1 - normative-drive-to-put-off-mask
    ]
  ]
  if action = "PUTTING OFF MASK"
  [
    calculate-a-salience "NORM: WEAR A MASK"
    calculate-a-salience "NORM: BREATHE FREELY"
    let np-do position "NORM: BREATHE FREELY" my-norms
    let np-dont position "NORM: WEAR A MASK" my-norms
    let normative-drive-to-put-off-mask item np-do my-norm-saliences
    let normative-drive-to-put-on-mask item np-dont my-norm-saliences
    ifelse normative-drive-to-put-off-mask > normative-drive-to-put-on-mask
    [
      set normative-drive normative-drive-to-put-off-mask
    ]
    [
      set normative-drive 1 - normative-drive-to-put-on-mask
    ]
  ]
end


;; calculates the individual drive to take a special action, used in the procedure propensity-to
to calculate-my-individual-drive-to [ action ]
  if not member? action all-actions
  [
    util-flog 0 ( word "calculate-my-individual-drive-to: " action " is not a valid action." )
    set ERROR-CONDITION? true
    stop
  ]
  let my-relative-risk-position 0
  if any? my-neighbours
  [
    let neighbours-to-compare-with ( turtle-set self my-neighbours )
    set my-relative-risk-position position my-risk sort [ my-risk ] of neighbours-to-compare-with / ( count neighbours-to-compare-with )
  ]
  if action = "PUTTING ON MASK" or action = "STAYING AT HOME"
  [
    set individual-drive my-relative-risk-position
  ]
  if action = "PUTTING OFF MASK"
  [
    set individual-drive ( 1 - my-relative-risk-position )
  ]
  if action = "GOING TO PARTY"
  [
    set individual-drive ( 1 - 0.5 * my-relative-risk-position )
  ]
end

;; calculates the salience of a norm from the contents of the agent's normative board (its normative memory)
;; the salience of a norm is calculated with a formula explained in the ODD (Appendix "Minding Norms in a Nutshell")
;; as this formula cannot be appropriately represented in this program code, its main components are commented below
to calculate-a-salience [ a-norm ]
  if not member? a-norm all-norms
  [
    util-flog 0 ( word "calculate-a-salience: " a-norm " is not a valid norm." )
    set ERROR-CONDITION? true
    stop
  ]
  let np position a-norm my-norms
;; C, V, OC, OV, P, S, EC and EV contain the numbers of the different kinds of norm invocations stored in
;; the normative memory of an agent- The differences of positive and negative invocations, divided by their
;; sums, are weighted with the coefficients WC .. WE. First each of the terms (C-V)/(C+V) etc. have to be
;; calculated. In parallel two constants have to be calculated (calle beta and one-divided-by-alpha) which
;; make sure that the formula yields a value between 0 and 1
  let C item np my-compliances
  let V item np my-violations
  let OC item np other-compliances
  let OV item np other-violations
  let P item np punishments
  let S item np sanctions
  let EC item np explicit-compliance-invocations
  let EV item np explicit-violations
  let beta 0
  let one-divided-by-alpha 0
;; the first term OWN=(C-V)/(C+V)
  let OWN 0
  if C + V > 0
  [
    set OWN ( C - V ) / ( C + V )
  ]
  set beta beta + WC
  set one-divided-by-alpha one-divided-by-alpha + 2 * WC
;; the second term OBS=(OC-OV)/(OC+OV)
  let OBS 0
  if OC + OV > 0
  [
    set OBS ( OC - OV ) / ( OC + OV )
  ]
  set beta beta + WO
  set one-divided-by-alpha one-divided-by-alpha + 2 * WO
;; the third term NPV=(OV+V-P-S)/(OV+V)
  let NPV 0
  if OV + V > 0
  [
    set NPV (OV + V - P - S )
    if NPV < 0 [ set NPV 0 ]
    set NPV NPV / ( OV + V)
  ]
  set beta beta + WNPV
  set one-divided-by-alpha one-divided-by-alpha + WNPV
;; the first part of the fourth term P_max=P/max(OV+V,P+S)
  let maxPSOV P + S
  if OV + V > P + S [ set maxPSOV OV + V ]
  let P_max 0
  if maxPSOV > 0
  [
    set P_max P / maxPSOV
  ]
  set one-divided-by-alpha one-divided-by-alpha + WP
;; the second part of the fourth term S_max/max(OV+V,P+S)
  let S_max 0
  if maxPSOV > 0
  [
    set S_max S / maxPSOV
  ]
  set one-divided-by-alpha one-divided-by-alpha + WS
;; the fifth term ECEV=(EC-EV)/(EC+EV)
  let ECEV 0
  if EC + EV > 0
  [
    set ECEV ( EC - EV ) / ( EC + EV )
  ]
  set beta beta + WE
  set one-divided-by-alpha one-divided-by-alpha + 2 * WE
;; calculate the weighted sum of the five terms
  let SAL 0
  if one-divided-by-alpha > 0
  [
    set SAL ( ( beta + OWN * WC + OBS * WO - NPV * WNPV + P_max * WP + S_max * WS + ECEV * WE ) / one-divided-by-alpha )
  ]
;; and checking whether the result is between 0 and 1
  if SAL < 0 or SAL > 1
  [
    util-flog -2 ( word "ERROR? " a-norm ": " ( list C V OC OV P S EC EV ) )
    util-flog -2 ( word "ERROR? " a-norm ": " ( list OWN OBS NPV P_max S_max ECEV beta one-divided-by-alpha SAL ) )
    set ERROR-CONDITION? true
  ]
  if member? self witnesses [ util-flog 20 ( word  ": salience of " a-norm " calculated as " precision SAL 4 ) ]
;; finally insert the calculated salience into at the respective place in the list of norm saliences for further use
  set my-norm-saliences replace-item np my-norm-saliences SAL
end

;; all kinds of agents update their individual event board (called by every update-... procedure)
to increase-invocation-counter [ i-item a-norm ]
  let this-norm position a-norm all-norms
  let this-sublist item this-norm invocation-count
  let this-subsublist item 1 this-sublist
  let this-entry item i-item this-subsublist
  set this-subsublist replace-item i-item this-subsublist ( this-entry + 1 )
  set this-sublist replace-item 1 this-sublist this-subsublist
  set invocation-count replace-item this-norm invocation-count this-sublist
end

;; the following procedures update the different fields of the normative board
to update-compliances [ a-norm ]
  increase-invocation-counter 0 a-norm
  if member? a-norm my-norms
  [
      let np position a-norm my-norms
      let old-value item np my-compliances
      set my-compliances replace-item np my-compliances ( old-value * DISCOUNT + 1 )
  ]
  ask my-neighbours [ update-observed-compliances a-norm ]
end

to update-violations [ a-norm ]
  increase-invocation-counter 1 a-norm
  if member? a-norm my-norms
  [
      let np position a-norm my-norms
      let old-value item np my-violations
      set my-violations replace-item np my-violations ( old-value * DISCOUNT + 1 )
  ]
  ask my-neighbours [ update-observed-violations a-norm ]
end

to update-observed-compliances [ a-norm ]
  increase-invocation-counter 2 a-norm
  if member? a-norm my-norms
  [
      let np position a-norm my-norms
      let old-value item np other-compliances
      set other-compliances replace-item np other-compliances ( old-value * DISCOUNT + 1 )
  ]
end

to update-observed-violations [ a-norm ]
  increase-invocation-counter 3 a-norm
  if member? a-norm my-norms
  [
      let np position a-norm my-norms
      let old-value item np other-violations
      set other-violations replace-item np other-violations ( old-value * DISCOUNT + 1 )
  ]
end

to update-punishments [ a-norm depth caller message-id ]
  if message-id = last-message-id
  [
    if member? self witnesses [ util-flog 0 ( word behaviorspace-run-number " UP: message id " message-id " already dealt with." ) ]
    stop
  ]
  increase-invocation-counter 4 a-norm
  if member? a-norm my-norms
  [
    let np position a-norm my-norms
    let old-value item np punishments
    set punishments replace-item np punishments ( old-value * DISCOUNT + 1 )
  ]
  if depth = 0
  [ ask my-neighbours
    [
      if member? a-norm my-norms
      [
        update-punishments a-norm 1 myself message-id
        update-explicit-compliance-invocations a-norm 1 myself message-id
      ]
    ]
  ]
  set last-message-id message-id
end

to update-sanctions [ a-norm depth caller message-id ]
  if message-id = last-message-id
  [
    if member? self witnesses [ util-flog 0 ( word behaviorspace-run-number " US: message id " message-id " already dealt with." ) ]
    stop
  ]
  increase-invocation-counter 5 a-norm
  if member? a-norm my-norms
  [
    let np position a-norm my-norms
    let old-value item np sanctions
    set sanctions replace-item np sanctions ( old-value * DISCOUNT + 1 )
  ]
  if depth = 0
  [ ask my-neighbours
    [
      if member? a-norm my-norms
      [
        update-sanctions a-norm 1 myself message-id
        update-explicit-compliance-invocations a-norm 1 myself message-id
      ]
    ]
  ]
  set last-message-id message-id
end

to update-explicit-compliance-invocations [ a-norm depth caller message-id ]
  increase-invocation-counter 6 a-norm
  if member? a-norm my-norms
  [
    let np position a-norm my-norms
    let old-value item np explicit-compliance-invocations
    set explicit-compliance-invocations replace-item np explicit-compliance-invocations ( old-value * DISCOUNT + 1 )
  ]
  if depth = 0
  [ ask my-neighbours
    [
      if member? a-norm my-norms
      [
        update-sanctions a-norm 1 myself message-id
        update-explicit-compliance-invocations a-norm 1 myself message-id
      ]
    ]
  ]
  set last-message-id message-id
end

to update-explicit-violations [ a-norm depth caller message-id ]
  if message-id = last-message-id
  [
    if member? self witnesses [ util-flog 0 ( word behaviorspace-run-number " UEV: message id " message-id " already dealt with." ) ]
    stop
  ]
  increase-invocation-counter 7 a-norm
  if member? a-norm my-norms
  [
    let np position a-norm my-norms
    let old-value item np explicit-violations
    set explicit-violations replace-item np explicit-violations ( old-value * DISCOUNT + 1 )
  ]
  if depth = 0
  [
    let addressees ( my-neighbours ) with [ member? a-norm my-norms ]
    ask addressees
    [
      if member? a-norm my-norms
      [
        update-sanctions a-norm 1 myself message-id
        update-explicit-violations a-norm 1 myself message-id
      ]
    ]
  ]
  set last-message-id message-id
end

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; utilities
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; converts a number into a string with leading zeroes of the desired number of digits, used for setting the names of the directories for the run results
to-report format [ digits number ]
  let result ( word round number )
  while [ length result < digits ] [ set result ( word "0" result ) ]
  report result
end

;; to assign a shape according to health state
to-report state-shape
  let result "face neutral"
  if state = "dead" [ set result "Cross" ]
  if state = "susceptible" or state = "recovered" or state = "immune" [ set result "face happy" ]
  if state = "infected" or state = "ill" [ set result "face sad" ]
  report result
end


;; a wrapper for a time procedure, avoiding possible error situations
to time-schedule-days [ a-time ]
  if time:is-equal a-time TNOW [ stop ]
  if time:is-between a-time startdate TNOW [ stop ]
  if end-of-run? [ stop ]
  ifelse time:is-between a-time TNOW stopdate
  [
    time:schedule-event "observer" [ [] -> daily-update ] a-time
    util-flog-observer -2 ( word "; observer; daily-update;" T a-time )
  ]
  [
    stop
  ]
end

;; a wrapper for a time procedure, avoiding possible error situations
to time-schedule-event [ executor a-task a-task-name a-time  ]
  if time:is-equal a-time TNOW [ stop ]
  if time:is-between a-time startdate TNOW [ stop ]
  if end-of-run? [ stop ]
  ifelse time:is-between a-time TNOW stopdate
  [
    time:schedule-event executor a-task a-time
    if a-task-name = "move around" [ set future-moves future-moves + 1 ]
    if a-task-name = "lose immunity" [ set immunity-loss-scheduled? true ]
    util-flog 20 ( word "; " executor "; " state " " a-task-name "; " T a-time )
  ]
  [
    stop
  ]
end

;; just a shorthand for the precision function to make the output more easily legible
to-report MP [ value-list ]
  report map [ value -> precision value 1 ] value-list
end

;; implements a uniform distribution with arbitrary min and max
to-report random-float-in-range [ a-min a-max ]
  report a-min + random-float ( a-max - a-min )
end

;; implements the beta distribution
to-report random-beta [ p q ]
  let gamma1 random-gamma p 1
  let gamma2 random-gamma q 1
  let result gamma1 / ( gamma1 + gamma2 )
  report result
end

;; shorthand for the square of a number
to-report sqr [ a-number ]
  report a-number * a-number
end

;; reports the current logotime
to-report TNOW
  report time:plus startdate ticks "hours"
end

;; reports a string containing the current date and time
to-report T [ a-logotime ]
  ifelse a-logotime = NULL or a-logotime = 0
  [
    report a-logotime
  ]
  [
    report time:show a-logotime "yyyy-MM-dd HH:mm:ss"
  ]
end

;; allows the observer to write to the logfile
to util-flog-observer [ level something ]
  file-open logfile-name
  if level < log-level [ file-show ( word "; " level "; " something ) ]
  file-flush
end

;; allows an agent to write to the logfile
to util-flog [ level something ]
  file-open logfile-name
  if level < log-level or witness? [ file-show ( word "; " level "; " something ) ]
  file-flush
end

;; calculates the maximum height of a histogram
to-report max-freq [ ncat a-list ]
  let a-table map [ value -> ( round ( value * ncat ) ) / ncat ] a-list
  let mode max modes a-table
  report reduce
    [ [ m n ] -> ifelse-value ( n = mode ) [ m + 1 ] [ m ] ] ( fput 0 a-table )
end

to-report rectify-date
  let date-string date-and-time
  let year substring date-string 23 27
  let month substring date-string 19 22
  if month = "Okt" [ set month "Oct" ]
  if month = "Dez" [ set month "Dec" ]
  if month = "Mär" [ set month "Mar" ]
  if month = "Mai" [ set month "May" ]
  let day substring date-string 16 18
  let AMPM substring date-string 13 15
  let hour substring date-string 0 2
  let mmin substring date-string 3 5
  let sec substring date-string 6 12
  let month-number ( 1 + ( position month ( list "Jan" "Feb" "Mar" "Apr" "May" "Jun" "Jul" "Aug" "Sep" "Oct" "Nov" "Dec" ) ) )
  ifelse month-number < 10 [ set month ( word "0" month-number ) ] [ set month ( word month-number ) ]
  let num-hour read-from-string hour
  let num-min read-from-string mmin
  let num-sec read-from-string sec
  let time-of-day ( num-sec + 60 * num-min + 3600 * num-hour ) / 3600
  if ( AMPM = "PM" or AMPM = "pm" ) and time-of-day < 12 [ set hour ( word ( num-hour + 12 ) ) ]
  set date-string ( word year "-" month "-" day "-" hour "-" mmin "-" sec )
  report date-string
end
@#$#@#$#@
GRAPHICS-WINDOW
712
13
1507
809
-1
-1
7.8
1
5
1
1
1
0
0
0
1
-50
50
-50
50
1
1
1
ticks
30.0

BUTTON
10
10
90
45
setup
setup
NIL
1
T
OBSERVER
NIL
NIL
NIL
NIL
1

BUTTON
95
10
175
45
go
go
NIL
1
T
OBSERVER
NIL
NIL
NIL
NIL
0

SLIDER
10
92
190
125
initial-counties
initial-counties
2
13
13.0
1
1
NIL
HORIZONTAL

SLIDER
10
125
190
158
max-months
max-months
0
24
24.0
1
1
NIL
HORIZONTAL

SWITCH
469
202
625
235
auto-lockdown?
auto-lockdown?
1
1
-1000

SLIDER
469
234
626
267
lockdown-at
lockdown-at
0
200
50.0
10
1
NIL
HORIZONTAL

SLIDER
469
267
626
300
lockdown-end-at
lockdown-end-at
0
200
25.0
5
1
NIL
HORIZONTAL

SLIDER
10
169
190
202
far-commuter
far-commuter
0
1
0.2
0.05
1
NIL
HORIZONTAL

SLIDER
10
202
190
235
short-distance
short-distance
0
10
2.0
1
1
NIL
HORIZONTAL

SLIDER
10
235
190
268
long-distance
long-distance
0
100
50.0
1
1
NIL
HORIZONTAL

SLIDER
10
274
260
307
distance-for-infection
distance-for-infection
0
3
2.25
0.25
1
NIL
HORIZONTAL

SLIDER
10
307
260
340
infections-per-contact
infections-per-contact
0
1
0.25
0.25
1
NIL
HORIZONTAL

SLIDER
10
340
260
373
tested-per-infected
tested-per-infected
0
1
1.0
0.05
1
NIL
HORIZONTAL

SLIDER
10
373
260
406
asymptomatic-per-infected
asymptomatic-per-infected
0
1
0.15
0.05
1
NIL
HORIZONTAL

SLIDER
10
407
260
440
case-fatality-rate
case-fatality-rate
0
1
0.03
0.01
1
NIL
HORIZONTAL

SLIDER
9
487
189
520
time-to-infectiosity-min
time-to-infectiosity-min
0
6
1.0
1
1
NIL
HORIZONTAL

SLIDER
189
487
369
520
time-to-infectiosity-max
time-to-infectiosity-max
time-to-infectiosity-min
15
4.0
1
1
NIL
HORIZONTAL

SLIDER
9
519
189
552
incubation-time-min
incubation-time-min
0
20
5.0
1
1
NIL
HORIZONTAL

SLIDER
189
519
369
552
incubation-time-max
incubation-time-max
incubation-time-min
20
10.0
1
1
NIL
HORIZONTAL

SLIDER
9
553
189
586
illness-duration-min
illness-duration-min
0
30
12.0
1
1
NIL
HORIZONTAL

SLIDER
189
553
369
586
illness-duration-max
illness-duration-max
illness-duration-min
60
17.0
1
1
NIL
HORIZONTAL

SLIDER
9
585
189
618
time-till-immunity-min
time-till-immunity-min
0
20
0.0
1
1
NIL
HORIZONTAL

SLIDER
189
585
369
618
time-till-immunity-max
time-till-immunity-max
time-till-immunity-min
30
2.0
1
1
NIL
HORIZONTAL

SLIDER
9
618
189
651
immunity-duration-min
immunity-duration-min
0
24
24.0
1
1
NIL
HORIZONTAL

SLIDER
189
618
369
651
immunity-duration-max
immunity-duration-max
immunity-duration-min
72
30.0
3
1
NIL
HORIZONTAL

CHOOSER
9
688
189
733
zero-person
zero-person
"densest" "sparsest" "center" "random"
0

SWITCH
9
733
189
766
interruptible?
interruptible?
0
1
-1000

PLOT
1522
13
1912
177
Current state of the population 1
time (days)
per 100,000
0.0
10.0
0.0
1000.0
true
true
"" "set-plot-y-range 0 100000"
PENS
"susceptible" 1.0 0 -7500403 true "" "plotxy round ( ticks / 24 ) count people with [ state = \"susceptible\" ] * 100000 / total-population"
"immune" 1.0 0 -13840069 true "" "plotxy ( ticks / 24 ) count people with [ state = \"immune\" ] * 100000 / total-population"
"all-ever-infected" 1.0 0 -6459832 true "" "plotxy ( ticks / 24 ) all-ever-infected * 00 / total-population"

PLOT
1530
173
1874
333
Current state of the population 2
time (days)
per 100.000
0.0
10.0
0.0
1.0
true
true
"" ""
PENS
"infected" 1.0 0 -2674135 true "" "plotxy ( ticks / 24 ) count people with [ state = \"infected\" ] * 100000 / total-population"
"ill" 1.0 0 -10141563 true "" "plotxy ( ticks / 24 ) count people with [ state = \"ill\" ] * 100000 / total-population"
"recovered" 1.0 0 -11221820 true "" "plotxy ( ticks / 24 ) count people with [ state = \"recovered\" ] * 100000 / total-population"
"dead" 1.0 0 -16777216 true "" "plotxy ( ticks / 24 ) count people with [ state = \"dead\" ] * 100000 / total-population"
"infectious" 1.0 0 -955883 true "" "plotxy ( ticks / 24 ) count people with [ infectious? ] * 1000 / total-population"

PLOT
1530
333
1866
493
Infections in last 24 hours
time (days)
per 100,000
0.0
10.0
0.0
0.1
true
true
"" ""
PENS
"real" 1.0 0 -16777216 true "" "plotxy ( ticks / 24 ) newly-infected * 100000 / total-population"
"reported" 1.0 0 -2674135 true "" "plotxy ( ticks / 24 ) newly-infected-reported * 100000 / total-population"
"mov aver" 1.0 0 -11221820 true "" "if length newly-infected-7-days = 7 [ plotxy ( ticks / 24 - 3 ) newly-infected-7-days-ma  * 100000 / total-population ]"

PLOT
1530
493
1866
653
Deaths in last 24 hours
time (days)
per 100,000
0.0
10.0
0.0
0.1
true
true
"" ""
PENS
"real" 1.0 0 -16777216 true "" "plotxy ( ticks / 24 ) newly-deceased  * 100000 / total-population"
"reported" 1.0 0 -2674135 true "" "plotxy ( ticks / 24 ) newly-deceased-reported * 100000 / total-population"
"mov aver" 1.0 0 -11221820 true "" "if length newly-deceased-7-days = 7 [ plotxy ( ticks / 24 - 3 ) newly-deceased-7-days-ma * 100000 / total-population ]"

PLOT
1530
653
1857
813
R0 real / est
time (days)
R0
0.0
1.0
0.0
7.0
true
true
"" "set-plot-y-range 0 7"
PENS
"R0 real" 1.0 0 -16777216 true "" "if any? people with [ infectious? ] [ plotxy ( ticks / 24 ) reproduction-number ]"
"kappa" 1.0 0 -13840069 true "" "ifelse ( count people with [ infectious? ] > 1 ) [ plotxy ( ticks / 24 ) dispersion-parameter ][ plotxy (ticks / 24 ) 0 ]"
"1.0" 1.0 0 -955883 true "" "plotxy ( ticks / 24 ) 1.0"
"R0 est 4" 1.0 0 -13345367 true "" "ifelse length newly-infected-4-days-ma-5-days = 5 and last newly-infected-4-days-ma-5-days > 0 [ plotxy ( ticks / 24 ) first newly-infected-4-days-ma-5-days / last newly-infected-4-days-ma-5-days ] [ plotxy ( ticks / 24 ) 0 ]"
"R0 est 7" 1.0 0 -13791810 true "" "ifelse length newly-infected-7-days-ma-5-days = 5 and last newly-infected-7-days-ma-5-days > 0 [ plotxy ( ticks / 24 ) first newly-infected-7-days-ma-5-days / last newly-infected-7-days-ma-5-days ] [ plotxy ( ticks / 24 ) 0 ]"

MONITOR
177
10
362
55
Date and time
( word item ( time:get \"dayofweek\" T TNOW - 1 ) day-names \" \" T TNOW )
17
1
11

MONITOR
8
843
157
892
ERROR-CONDITION?
ERROR-CONDITION?
17
1
12

MONITOR
9
967
2302
1016
county statistics (area density population infectious)
sort-by [ [?1 ?2] -> item 0 ?1 < item 0 ?2 ] [ ( list who substring colour 0 3 area precision density 2 population count my-people with [ infectious? ] ) ] of counties
17
1
12

CHOOSER
193
64
378
109
density-mode
density-mode
"all counties equal" "linearly decreasing" "d~1/area"
2

MONITOR
1918
13
2008
50
susceptible
count people with [ state = \"susceptible\" ]
17
1
9

MONITOR
1918
53
2008
90
infected
count people with [ state = \"infected\" ]
17
1
9

MONITOR
1918
93
2008
130
ill
count people with [ state = \"ill\" ]
17
1
9

MONITOR
1918
133
2008
170
recovered
count people with [ state = \"recovered\" ]
17
1
9

MONITOR
1918
173
2008
210
immune
count people with [ state = \"immune\" ]
17
1
9

MONITOR
1918
213
2008
250
dead
count people with [ state = \"dead\" ]
17
1
9

MONITOR
2008
13
2098
50
inf/sus
count people with [ state = \"susceptible\" and infectious? ]
17
1
9

MONITOR
2008
53
2098
90
inf/inf
count people with [ state = \"infected\" and infectious? ]
17
1
9

MONITOR
2008
93
2098
130
inf/ill
count people with [ state = \"ill\" and infectious? ]
17
1
9

MONITOR
2008
133
2098
170
inf/rec
count people with [ state = \"recovered\" and infectious? ]
17
1
9

MONITOR
2008
173
2098
210
inf/imm
count people with [ state = \"immune\" and infectious? ]
17
1
9

MONITOR
2008
213
2098
250
inf/dead
count people with [ state = \"dead\" and infectious? ]
17
1
9

MONITOR
2008
253
2098
290
infectious
count people with [ infectious? ]
17
1
9

MONITOR
162
893
304
942
scheduled events
time:size-of-schedule
17
1
12

MONITOR
369
10
553
55
stop date
( word item ( time:get \"dayofweek\" T day-before-stopdate - 1 ) day-names \" \" T day-before-stopdate )
17
1
11

MONITOR
1875
657
1930
702
R0(t)
reproduction-number
3
1
11

MONITOR
305
844
494
893
NIL
newly-infected-7-days
17
1
12

MONITOR
305
893
495
942
NIL
newly-deceased-7-days
17
1
12

MONITOR
560
11
706
56
NIL
period
17
1
11

MONITOR
1874
337
1949
374
new infected
newly-infected
0
1
9

MONITOR
1874
369
1949
406
reported
newly-infected-reported
17
1
9

MONITOR
1874
403
1949
440
mov average
newly-infected-7-days-ma
2
1
9

MONITOR
1919
253
2009
290
all ever
all-ever-infected
0
1
9

OUTPUT
317
314
701
483
9

MONITOR
1874
495
1949
532
new deceased
newly-deceased
17
1
9

MONITOR
1874
528
1949
565
reported
newly-deceased-reported
17
1
9

MONITOR
1874
560
1949
597
mov average
newly-deceased-7-days-ma
2
1
9

TEXTBOX
635
227
705
302
reported\ninfections \nper 100,000\npopulation\nduring past\nseven days
9
0.0
1

SLIDER
8
654
188
687
reporting-delay
reporting-delay
0
1
0.0
0.05
1
NIL
HORIZONTAL

PLOT
379
485
539
605
conviv*carel su
NIL
NIL
0.0
1.0
0.0
10.0
true
false
"set-plot-pen-mode 1\nset-histogram-num-bars 20\nif any? people with [ state = \"susceptible\" ] [ set-plot-y-range 0 max-freq-ccs ]" "set-plot-pen-mode 1\nset-histogram-num-bars 20\nif any? people with [ state = \"susceptible\" ] [ set-plot-y-range 0 max-freq-ccs ]"
PENS
"default" 1.0 0 -16777216 true "" "histogram [ conviviality * carelessness ] of people with [ state = \"susceptible\" ]"
"pen-1" 1.0 0 -2674135 true "" "if any? people [ plot-pen-reset plot-pen-up plotxy median [ conviviality * carelessness ] of people with [ state = \"susceptible\" ] 0 plot-pen-down plotxy median [ conviviality * carelessness ] of people with [ state = \"susceptible\" ] max-freq-ccs ]"

PLOT
539
485
699
605
conviv*carel in
NIL
NIL
0.0
1.0
0.0
10.0
true
false
"set-plot-pen-mode 1\nset-histogram-num-bars 20\nif any? people with [ state != \"susceptible\" ] [ set-plot-y-range 0 max-freq-cci ]" "set-plot-pen-mode 1\nset-histogram-num-bars 20\nif any? people with [ state != \"susceptible\" ] [ set-plot-y-range 0 max-freq-cci ]"
PENS
"default" 1.0 0 -16777216 true "" "histogram [ conviviality * carelessness ] of people with [ state != \"susceptible\" ]"
"pen-1" 1.0 0 -2674135 true "" "if any? people [ plot-pen-reset plot-pen-up plotxy median [ conviviality * carelessness ] of people with [ state != \"susceptible\" ] 0 plot-pen-down plotxy median [ conviviality * carelessness ] of people with [ state != \"susceptible\" ] max-freq-cci ]"

PLOT
380
668
540
788
conviv*carel*perc
NIL
NIL
0.0
0.4
0.0
10.0
true
false
"set-plot-pen-mode 1\nset-histogram-num-bars 20\nif any? people [ set-plot-x-range 0 max list 0.1 ( ( ceiling ( max [ conviviality * carelessness * ( 1 - perceived-risk ) ] of people * 1000 ) ) / 1000 ) set-plot-y-range 0 max-freq-ccp  ]" "set-plot-pen-mode 1\nset-histogram-num-bars 20\nif any? people [ set-plot-x-range 0 max list 0.1 ( ( ceiling ( max [ conviviality * carelessness * ( 1 - perceived-risk ) ] of people * 1000 ) ) / 1000 ) set-plot-y-range 0 max-freq-ccp  ]"
PENS
"default" 1.0 0 -16777216 true "" "histogram [ conviviality * carelessness * ( 1 - perceived-risk ) ] of people"
"pen-1" 1.0 0 -2674135 true "" "if any? people [ plot-pen-reset plot-pen-up plotxy median [ conviviality * carelessness * ( 1 - perceived-risk ) ] of people 0 plot-pen-down plotxy median [ conviviality * carelessness * ( 1 - perceived-risk ) ] of people max-freq-ccp  ]"

PLOT
540
668
700
788
perc inf risk
NIL
NIL
0.0
0.4
0.0
19.0
true
false
"set-plot-pen-mode 1\nset-histogram-num-bars 20" "set-plot-pen-mode 1\nset-histogram-num-bars 20\nif any? people [ set-plot-x-range 0 max list 0.1 ( ( ceiling ( max [ perceived-risk ] of people * 1000 ) ) / 1000 ) set-plot-y-range 0 max-freq-pir ]"
PENS
"default" 1.0 0 -16777216 true "" "histogram [ perceived-risk ] of people"
"pen-1" 1.0 0 -2674135 true "" "if any? people [ plot-pen-reset plot-pen-up plotxy median [ perceived-risk ] of people 0 plot-pen-down plotxy median [ perceived-risk ] of people max-freq-pir ]"

MONITOR
162
843
304
892
past events
past-events-counter
17
1
12

MONITOR
620
794
700
839
max
max [ perceived-risk ] of people
4
1
11

MONITOR
1874
700
2050
745
R0 MA 4
map [ ?1 -> precision ?1 1 ] newly-infected-4-days-ma-5-days
17
1
11

MONITOR
1874
747
2051
792
R0 MA 7
map [ ?1 -> precision ?1 1 ] newly-infected-7-days-ma-5-days
17
1
11

PLOT
540
839
700
959
med perc risk
time
NIL
0.0
10.0
0.0
0.001
true
false
"" "set-plot-y-range 0 ifelse-value any? people with [ perceived-risk > 0.001 ][ precision ( max [ perceived-risk ] of people ) 3 ] [ 0.001 ]"
PENS
"default" 1.0 0 -16777216 true "" "if any? people [ plotxy ( ticks / 24 ) median [ perceived-risk ] of people ]"
"pen-1" 1.0 0 -7500403 true "" "if any? people [ plotxy ( ticks / 24 ) max [ perceived-risk ] of people ]"

MONITOR
222
749
350
794
NIL
pandemic-grows?
17
1
11

MONITOR
8
895
153
944
moves
( word min [ future-moves ] of people \" | \" count people with [ future-moves < 0 ] \" / \" count people with [ future-moves = 0 ] \" / \" count people with [ future-moves > 0 ] \" | \" max [ future-moves ] of people)
17
1
12

SWITCH
190
654
369
687
constant-seed?
constant-seed?
0
1
-1000

MONITOR
540
794
619
839
median
median [ perceived-risk ] of people
4
1
11

MONITOR
380
605
443
650
median
median [ conviviality * carelessness ] of people with [ state = \"susceptible\" ]
4
1
11

MONITOR
540
605
603
650
median
median [ conviviality * carelessness ] of people with [ state != \"susceptible\" ]
4
1
11

MONITOR
478
622
541
667
median
median [ conviviality * carelessness * ( 1 - perceived-risk ) ] of people
4
1
11

MONITOR
638
623
701
668
median
median [ perceived-risk ] of people
4
1
11

SLIDER
10
442
182
475
reporting-rate
reporting-rate
0
1
0.8
0.05
1
NIL
HORIZONTAL

CHOOSER
193
170
357
215
infectious-from-abroad
infectious-from-abroad
"none" "months 2..11" "months 7..8" "months 9..11"
0

SWITCH
394
66
573
99
normative-mode?
normative-mode?
0
1
-1000

SLIDER
572
66
705
99
NDW
NDW
0
1
0.8
0.05
1
NIL
HORIZONTAL

SLIDER
535
104
707
137
BACKGROUND
BACKGROUND
-10
10
-2.0
1
1
NIL
HORIZONTAL

MONITOR
2099
627
2194
672
mask wearers
count people with [ wears-a-mask? ]
17
1
11

MONITOR
2195
627
2307
672
staying at home
count people with [ stays-at-home? ]
17
1
11

PLOT
2105
13
2305
163
sal WEAR A MASK
NIL
NIL
0.0
1.0
0.0
10.0
true
false
"set-plot-pen-mode 1\nset-histogram-num-bars 20\nif any? people [ set-plot-x-range 0 1 set-plot-y-range 0 max-freq-ns  ]" "set-plot-pen-mode 1\nset-histogram-num-bars 20\nif any? people [ set-plot-x-range 0 1 set-plot-y-range 0 max-freq-ns  ]"
PENS
"default" 1.0 0 -16777216 true "" "if normative-mode? [ histogram [ item 0 my-norm-saliences ] of people ]"

PLOT
2105
164
2305
314
sal BREATHE FREELY
NIL
NIL
0.0
1.0
0.0
10.0
true
false
"set-plot-pen-mode 1\nset-histogram-num-bars 20\nif any? people [ set-plot-x-range 0 1 set-plot-y-range 0 max-freq-ns  ]" "set-plot-pen-mode 1\nset-histogram-num-bars 20\nif any? people [ set-plot-x-range 0 1 set-plot-y-range 0 max-freq-ns  ]"
PENS
"default" 1.0 0 -16777216 true "" "if normative-mode? [ histogram [ item 1 my-norm-saliences ] of people ]"

PLOT
2105
318
2305
468
sal STAY AT HOME
NIL
NIL
0.0
1.0
0.0
10.0
true
false
"set-plot-pen-mode 1\nset-histogram-num-bars 20\nif any? people [ set-plot-x-range 0 1 set-plot-y-range 0 max-freq-ns  ]" "set-plot-pen-mode 1\nset-histogram-num-bars 20\nif any? people [ set-plot-x-range 0 1 set-plot-y-range 0 max-freq-ns  ]"
PENS
"default" 1.0 0 -16777216 true "" "if normative-mode? [ histogram [ item 2 my-norm-saliences ] of people ]"

PLOT
2107
472
2307
622
sal GO TO PARTY
NIL
NIL
0.0
1.0
0.0
10.0
true
false
"set-plot-pen-mode 1\nset-histogram-num-bars 20\nif any? people [ set-plot-x-range 0 1 set-plot-y-range 0 max-freq-ns  ]" "set-plot-pen-mode 1\nset-histogram-num-bars 20\nif any? people [ set-plot-x-range 0 1 set-plot-y-range 0 max-freq-ns  ]"
PENS
"default" 1.0 0 -16777216 true "" "if normative-mode? [ histogram [ item 3 my-norm-saliences ] of people ]"

PLOT
2064
677
2309
875
norm actions
time (days)
per 100,000
0.0
10.0
0.0
10.0
true
true
"" ""
PENS
"mask" 1.0 0 -13791810 true "" "plotxy ( ticks / 24 )  count people with [ wears-a-mask? ] / total-population * 100000"
"at home" 1.0 0 -13840069 true "" "plotxy ( ticks / 24 )  count people with [ stays-at-home? ] / total-population * 100000"

PLOT
1860
794
2060
944
whom-I-infected
NIL
NIL
0.0
4.0
0.0
1.0
true
false
"set-plot-pen-mode 1" "set-plot-pen-mode 1\nset-plot-x-range 0 ifelse-value any? people with [ infectious? and length whom-I-infected > 0 ] [ max [ length whom-I-infected ] of people with [ infectious? ] ] [ 2 ]"
PENS
"default" 1.0 0 -16777216 true "" "histogram [ length whom-I-infected ] of people with [ infectious? ]"

MONITOR
264
268
419
313
mean near neighb dist
mean-distance-to-nearest-neighbour
6
1
11

CHOOSER
10
797
149
842
heterogeneity
heterogeneity
"none" "wrt risk" "wrt norm" "both"
3

MONITOR
718
863
851
908
max daily infected
map [ ?1 -> precision ?1 1 ] ( replace-item 0 newly-infected-7-days-ma-max (( item 0 newly-infected-7-days-ma-max ) / 24 ))
1
1
11

MONITOR
858
862
971
907
max daily dead
map [ ?1 -> precision ?1 1 ] replace-item 0 newly-deceased-7-days-ma-max ( ( item 0 newly-deceased-7-days-ma-max ) / 24 )
17
1
11

PLOT
1532
817
1859
967
test positivity rate
NIL
NIL
0.0
10.0
0.0
0.001
true
true
"" "set-plot-y-range 0 max ( list 0.001 max-positive-test-rate )"
PENS
"random" 1.0 0 -16777216 true "" "plotxy ( ticks / 24 ) random-positive-test-rate"
"suspect" 1.0 0 -2674135 true "" "plotxy ( ticks / 24 ) suspect-positive-test-rate"

SLIDER
10
55
193
88
total-population
total-population
0
100000
10000.0
5000
1
NIL
HORIZONTAL

MONITOR
1009
864
1114
909
incidence rate
round sum newly-infected-7-days / total-population * 100000
0
1
11

MONITOR
1990
658
2055
703
1+1/k(t)
dispersion-parameter
3
1
11

MONITOR
1930
658
1988
703
var
var-infected
17
1
11

PLOT
1329
818
1529
968
whom-I-infected_t
NIL
NIL
0.0
10.0
0.0
1.0
true
true
"" ""
PENS
"< 1" 1.0 0 -16777216 true "" "if any? infectious-people [ plotxy ( ticks / 24 ) ( count infectious-people with [ length whom-I-infected < 1 ] / count infectious-people ) ]"
"< 2" 1.0 0 -7500403 true "" "if any? infectious-people [ plotxy ( ticks / 24 ) ( count infectious-people with [ length whom-I-infected < 2 ] / count infectious-people ) ]"
"< 3" 1.0 0 -2674135 true "" "if any? infectious-people [ plotxy ( ticks / 24 ) ( count infectious-people with [ length whom-I-infected < 3 ] / count infectious-people ) ]"
"< 4" 1.0 0 -955883 true "" "if any? infectious-people [ plotxy  ( ticks / 24 ) ( count infectious-people with [ length whom-I-infected < 4 ] / count infectious-people ) ]"
"< 5" 1.0 0 -6459832 true "" "if any? infectious-people [ plotxy ( ticks / 24 ) ( count infectious-people with [ length whom-I-infected < 5 ] / count infectious-people ) ]"
"< 6" 1.0 0 -16777216 true "" "if any? infectious-people [ plotxy ( ticks / 24 ) ( count infectious-people with [ length whom-I-infected < 6 ] / count infectious-people ) ]"
"< 7" 1.0 0 -7500403 true "" "if any? infectious-people [ plotxy ( ticks / 24 ) ( count infectious-people with [ length whom-I-infected < 7 ] / count infectious-people ) ]"
"< 8" 1.0 0 -2674135 true "" "if any? infectious-people [ plotxy ( ticks / 24 ) ( count infectious-people with [ length whom-I-infected < 8 ] / count infectious-people ) ]"
"< 9" 1.0 0 -955883 true "" "if any? infectious-people [ plotxy  ( ticks / 24 ) ( count infectious-people with [ length whom-I-infected < 9 ] / count infectious-people ) ]"
"all" 1.0 0 -6459832 true "" "if any? infectious-people [ plotxy ( ticks / 24 ) ( count infectious-people with [ length whom-I-infected < 100 ] / count infectious-people ) ]"

@#$#@#$#@
## WHAT IS IT?

This is an event-oriented model of an epidemic in which agents move around, eventually infect each other, fall ill, recover, become immune or die from the disease. Agents sense how many infectious other agents are in their neighbourhood and all over the population and calculate their personal risk of being infected depending on their own (constant) conviviality and carelessness and in their (variable) risk perception,

In the norm-oriented version the also communicate about

  * wearing masks (as opposed to breathe freely) which reduces their probability of being infected by the factor 0.2 (can be changed in the code) and
  * staying at home (instead of going to places where they can be infected) which reduces their probability of being infected to 0.

Switching between either behaviours corroborates slightly the propensity to keep on with the new behaviour; neighbours are encouraged to behave in the same manner.

## HOW IT WORKS

The model covers an area of 10,201 patches, divided in several (up to 13) regions (`counties`) with the same or different population density.

When the model starts, exactly one agent in the county with the highest density (alternatively: with the smallest density, or at a random patch) is infected.

At the very beginning all agents are scheduled to move a short distance within some short time. If at the end of this move they encounter an infectiuos agent they encounter the risk of getting infected,  continue to move around, become infectious themselves, fall ill with a certain probability, go to hospital and after they recovered or died return to their start patch. If they recovered they continue to move, lose their infectiosity after some time, become immune for a limited (but rather long) period, and if they lose their immunity they become susceptive again.

Infections are reported with some delay, but at the very latest when they fall ill and are brought to hospital.

The individual risk of being infected depends, of course, on how many infectious agents are in the neighbourhood (defined by `distance-for-infection`, but also on three attributes of the individual agents:

  * `conviviality`, a beta distributed instance variable between 0 and 1 initialised in the setup and kept constant
  * `carelessness` , another beta distributed instance variable between 0 and 1 initialised in the setup and kept constant
  * `perceived-risk` which at any time is either the percentage of infectious agents within a radius of 5 (while the epidemic increases) or the overall percentage of infectious agents (otherwise).

`conviviality' and 'carelessness` can alternatively be kept identical for all people when the `heterogeneity` switch is set to `both` or `wrt risk`.

The risk of being infected is proportional to the product of the global constant `infections-per-contact`, and the individual attributes `conviviality`, `carelessness` and `( 1 - perceived-risk )`.

If the normative mode is switched on, all agents are initialised with a memory of past norm invocations (for the theory behind norm invocations and norm-dependent behaviour see Conte et al. 2014 and Elsenbroich et al. 2016). The number of such norm invocations depends on the global variable `BACKGROUND` (-10 .. +10) which determines the mean of the number of invocations of norms relating to a fear of the pandemic and the fear of losing one's freedom (the individual numbers are either equal to BACKGROUND or beta distributed around this mean with a range of 20, depending on the `heterogeneity` parameter). The norms are the following:

  * `FEAR OF PANDEMIC` related:
    * `NORM: STAY AT HOME`
    * `NORM: WEAR A MASK`
  * `FEAR OF OPPRESSION` related:
    * `NORM: GO TO PARTY`
    * `NORM: PUT OFF MASK`

Agents calculate the salience of these norm and the propensity to take repsective actions:

  * `PUTTING ON MASK`
  * `PUTTING OFF MASK`
  * `STAYING AT HOME`
  * `GOING TO PARTY`

The respective propensity depends on both the _individual drive_ and the _normative drive_. The former depends only on the perceived risk (and is the same as in the non-normative mode), whereas the latter depends on the current salience of the four norms.

Plots and monitors keep track of the epidemic. All of them are described in more detail in the ODD+D description accompanying this model.

The R0 plot keeps track of three variants of the _basic reproduction number R<sub>0</sub>_; the "real" reproduction number is calculated as the mean of the length of the infection chain of all currently infectious agents, the two others are calculated according to the rules published by the German Robert Koch Institute, in two variants, one using 4-day moving averages, the other uses 7-day moving averages of daily new infections, _R<sub>0</sub>_ being calculated as the quotient of the current moving average and the moving average four days earlier, hence _MA<sub>t</sub>/MA<sub>t-4</sub>_. See https://www.rki.de/DE/Content/Infekt/EpidBull/Archiv/2020/Ausgaben/17_20.pdf, p. 10--16.

## HOW TO USE IT

The buttons, sliders, switches etc. have the following meaning (from top left to bottom right, left of the view):

  * `setup` and `go` have the usual meaning, but in an event-oriented model `go` is a _go forever_ and can either be stopped with Tools->Halt or via an interruption mechanism explained below.
  * _Date and time_ gives the simulated time which starts atthe beginning of the year 2020.
  * _stop date_ shows how long the simulation can run (namely for `max-months` months, if it is not interrupted before or ends when no infectious agents are left.
  * `total-population` sets the exact number of people spread over the counties according to the `density-mode` parameter with equal or different densities among the counties.
  * `initial-counties` determines how many subregions (`counties`) are displayed.
  * `density-mode` allows for three different modes of populating the `counties`: all with the same population density, with linearly decreasing density and with inversely decreasing as a function of the number of patches; a patch can sprout up to four agents — alternatively: nine agents — (except on the boundary of a neighbouring `county` where there is one or no agent per patch.
  * `infected-from-abroad` determines whether it is possible that some agents are infected from abroad (as if they had just returned from holidays abroad); in this case about two percent of the population become infected and infectious from the second to the eleventh simulated month (such an event is logged in the output area).
  * `period` shows whether the model is in lockdown mode or in normal mode; this can be switched automatically or by the user.
  * `max-months` determines the maximum duration of the simulation run.
  * `far-commuter` determines the percentage of agents which can move up to `long-distance` patches at a time.
  * `short-distance` determines how far (at most) the ordinary agents can walk at a time.
  * `long-distance` determines how far (at most) the far-commuting agents can walk at a time.
  * `auto-lockdown?` determines whether the model can enter the lockdown mode automatically. "Lockdown means" that the distances up to which agents may move are lowered to one half or its normal va,ue.
  * `lockdown-at` determines the limit for entering the lockdown: when the incidence rate (newly infectious agents in the paast seven days per 100,000) becomes greater than this limit, lockdown starts at the next midnight.
  * `lockdown-end-at` determines the limit for exiting the lockdown: when the incidence rate falls below this limit, lockdown ends at the next midnight.
  * `distance-for-infection` is the maximum distance within which infections are possible.
  * `infections-per-contact` is the ratio between successful infections and contacts.
  * `tested-per-infected` is the ratio between tested (and, hence, reportable) infections and all infections.
  * `asymptomatic-per-infected` is the ratio between the number of agents ehich show symptoms resulting in being hospitalised.
  * `case-fatality-rate` is the ratio between all deceased and all infected.
  * `time-to-infectiosity-min` and `-max` is the range of an uniform distribution for the time between being infected and being infectious (in days).
  * `incubation-time-min` and `-max`  is the range of an uniform distribution for the time between between being infected and showing symptoms of illness that lead to hospitalisation (in days).
  * 'illness-duration-min` and `-max` is the range of an uniform distribution for the time between falling ill and recovering or death (in days).
  * `immunity-duration-min` and `-max` is the range of an uniform distribution for the time between recovering and becoming susceptible after loss of immunity (in months).
  * `reporting-delay` is the upper end of the range of an uniform distribution determining when a test result is reported; over the weekend further delays are programmed (see code).
  * `zero-person` determines whether `person-zero` occurs in the `"densest"`, `"sparsest"` or a `"random" county`.
  * `interruptible?` determines whether the simulation run can be interrupted at the end of each simulated day. The switch can be changed at any time, and at midnight of the next simulated day a chooser opens which allows the user to determine how the simulation should go on. The options are
    * `"continue"` until next midnight;
    * `"go on forever"`; continue forever;
    * `"print profile and continue forever"`: (only for debugging purposes) print a profile (duration of all procedures) and continue;
    * `"halt and print profile and result!"`: end the simulation run and output all collected information;
    * `"spread"`:  `move-an-infectious-far-away`, i.e. select an infectious agent randomly and put it on a random patch;
    * `"normal time"`: exit from a user-defined lockdown;
    * `"lockdown"`: start a user-defined lockdown;
    * `"inspect"` a randomly selected infectious agent.

The followingng events can be scheduled by the model:

  * `move-around`: the agent moves to another place
  * `set-infectious`: the agent is infected and becomes infectious
  * `report-a-positive-test`: the agents is reported to the observer as infected
  * `fall-ill`: the agent shows symptoms and is brought to hospital
  * `recover-or-die`: the agent either recovers or dies and is returned to its original patch
  * `report-a-death`: the death of the agent is reported to the observer
  * `become-immune`: the agent is no longer infectious nor susceptible
  * `lose-immunity`: the agent loses its immunity and is susceptible again

The monitors and plots are more or less self-explanatory:

  * (left of the view) the two plots `conviv*careless...` show the distribution and the median of the product of the two individual constant attributes `conviviality` and `carelessness` separately for the susceptible and for all other agents and the individual variable attribute `perceived-risk` which is at the same time the probability of being infected once an infectious contact happened.
  *  the plot `conviv*carel*perc...` shows the distribution and the median of the product of the two individual constant attributes `conviviality` and `carelessness` and the individual variable attribute `perceived-risk` which is at the same time the probability of being infected once an infectious contact happened.
  * `perc inf risk` shows the distribution and the mean of the individual variable attribute `perceived-risk` which while the moving-average of the dayly number of newly infected agents rises is the percentage of infectious agents within a radius of 5, otherwise it is the overall percentage of infectious agents.
  * `mean and max perc risk` is the maximum of this distribution.
  * `med perc risk` represents the maximum and the median of the distribution of the `perceived-risk` attribute over time.
  * (right of the view) `current state of the population`: these two plots represent several global percentages,
    * the upper plot containing the percentages of susceptible agents and of all who were ever infected,
    * the second plot containing the percebtages of currently infected, infectious, ill, recovered, immune  and deceased persons
separated for better legibility.
  * `infectious in last 24 hours` represents the number of newly infected, those whose test was reported and a 7-day moving average of the former.
  * `deaths in past 24 hours` represents the number of newly deceased, those which were reported and a moving average of the former.
  * `R0 real / est` represents the daily estimates of the basic reproduction number (see above).
  * (below the view) the day and number when the daily infections and deaths,respectively, were at their maximum.
  * the current incidence rate.
  * the number,colour, area, density, population and the number of currently infectious agents per county.



## THINGS TO NOTICE

In each run, the model creates a directory into which 15 files are written (the final state of the world, the plots, a profile, a logging file with details about interesting events, and a result file with details about the epidemic histories in the individual counties). At the end of every simulated day, a PNG file is written, and these PNG files can be converted to an AVI file using mencoder (NetLogo's movie feature is not used as not all video programmes can work with MOV files). The world and the PNG files are not written in behaviour space mode.

Which events are logged can be determined by changing the first parameter of the util-flog procedure and/or by setting the respective switch.


## THINGS TO TRY

Try out what happens with user lockdowns or with automatic lockdowns, and perhaps change the parameters determining what happens during a lockdown.

Change the algorithm which determines the different risk perception during periods of decreasing and increasing new infections, respectively.

Change the period within which _infections from abroad_ can occur (currently from the second until the end of the eleventh month or in months 7 to 8 or in months 9 to 11).

## EXTENDING THE MODEL

One possible extension would be to close the borders between adjacent `counties` by always calling the `go-home` procedure when an agent is on one of the patches which are members of the `my-borderline` patch set of the respective `county`.

Another possible extension could introduce separate lockdowns per county.

## NETLOGO FEATURES

NetLogo 5.3.1 is used instead of NetLogo 6 as the time extension which comes with NetLogo 6 accepts only anonymous procedures, and a former version made ample use of named procedures when scheduling new events. The final version of this model will be converted to NetLogo 6.

## RELATED MODELS

In 2020, a large number of epidemic models were created and published, Any listing would be incomplete these days. The final version of this model will contain such a list.

The normative mode is shaped along the lines of Conte et al. 2014 and Elsenbroich et al. 2016 (particularly the chapters written by Klaus G. Troitzsch in these books).

## CREDITS AND REFERENCES

This model was written by Klaus G. Troitzsch who planneds to use some of its results in an invited talk to be given at the Österreichische Wissenschaftstag in Baden bei Wien on October 20, 2020 (but due to the Covid-19 pandemic, this event was postponed to 2021). A paper will also be published soon. The ODD+D description can be found at https://www.comses.net/users/2399/ together with this model.

Conte, Rosaria, and Giulia Andrighetto and Marco Campennì, eds. (2014) Minding norms.  Oxford and New York: Oxford University Press

Elsenbroich, Corinna, and David Anzola and Nigel Gilbert, eds. (2016) Social Dimensions of Organised Crime. Cham: Springer
@#$#@#$#@
default
false
0
Polygon -7500403 true true 150 0 60 300 150 225 240 300

airplane
true
0
Polygon -7500403 true true 150 0 135 15 120 60 120 105 15 165 15 195 120 180 135 240 105 270 120 285 150 270 180 285 210 270 165 240 180 180 285 195 285 165 180 105 180 60 165 15

arrow
true
0
Polygon -7500403 true true 150 0 0 150 105 150 105 293 195 293 195 150 300 150

box
false
0
Polygon -7500403 true true 150 285 285 225 285 75 150 135
Polygon -7500403 true true 150 135 15 75 150 15 285 75
Polygon -7500403 true true 15 75 15 225 150 285 150 135
Line -16777216 false 150 285 150 135
Line -16777216 false 150 135 15 75
Line -16777216 false 150 135 285 75

bug
true
0
Circle -7500403 true true 96 182 108
Circle -7500403 true true 110 127 80
Circle -7500403 true true 110 75 80
Line -7500403 true 150 100 80 30
Line -7500403 true 150 100 220 30

building institution
false
0
Rectangle -7500403 true true 0 60 300 270
Rectangle -16777216 true false 130 196 168 256
Rectangle -16777216 false false 0 255 300 270
Polygon -7500403 true true 0 60 150 15 300 60
Polygon -16777216 false false 0 60 150 15 300 60
Circle -1 true false 135 26 30
Circle -16777216 false false 135 25 30
Rectangle -16777216 false false 0 60 300 75
Rectangle -16777216 false false 218 75 255 90
Rectangle -16777216 false false 218 240 255 255
Rectangle -16777216 false false 224 90 249 240
Rectangle -16777216 false false 45 75 82 90
Rectangle -16777216 false false 45 240 82 255
Rectangle -16777216 false false 51 90 76 240
Rectangle -16777216 false false 90 240 127 255
Rectangle -16777216 false false 90 75 127 90
Rectangle -16777216 false false 96 90 121 240
Rectangle -16777216 false false 179 90 204 240
Rectangle -16777216 false false 173 75 210 90
Rectangle -16777216 false false 173 240 210 255
Rectangle -16777216 false false 269 90 294 240
Rectangle -16777216 false false 263 75 300 90
Rectangle -16777216 false false 263 240 300 255
Rectangle -16777216 false false 0 240 37 255
Rectangle -16777216 false false 6 90 31 240
Rectangle -16777216 false false 0 75 37 90
Line -16777216 false 112 260 184 260
Line -16777216 false 105 265 196 265

butterfly
true
0
Polygon -7500403 true true 150 165 209 199 225 225 225 255 195 270 165 255 150 240
Polygon -7500403 true true 150 165 89 198 75 225 75 255 105 270 135 255 150 240
Polygon -7500403 true true 139 148 100 105 55 90 25 90 10 105 10 135 25 180 40 195 85 194 139 163
Polygon -7500403 true true 162 150 200 105 245 90 275 90 290 105 290 135 275 180 260 195 215 195 162 165
Polygon -16777216 true false 150 255 135 225 120 150 135 120 150 105 165 120 180 150 165 225
Circle -16777216 true false 135 90 30
Line -16777216 false 150 105 195 60
Line -16777216 false 150 105 105 60

car
false
0
Polygon -7500403 true true 300 180 279 164 261 144 240 135 226 132 213 106 203 84 185 63 159 50 135 50 75 60 0 150 0 165 0 225 300 225 300 180
Circle -16777216 true false 180 180 90
Circle -16777216 true false 30 180 90
Polygon -16777216 true false 162 80 132 78 134 135 209 135 194 105 189 96 180 89
Circle -7500403 true true 47 195 58
Circle -7500403 true true 195 195 58

circle
false
0
Circle -7500403 true true 0 0 300

circle 2
true
8
Circle -16777216 true false 0 0 300
Circle -16777216 true false 30 30 240

cow
false
0
Polygon -7500403 true true 200 193 197 249 179 249 177 196 166 187 140 189 93 191 78 179 72 211 49 209 48 181 37 149 25 120 25 89 45 72 103 84 179 75 198 76 252 64 272 81 293 103 285 121 255 121 242 118 224 167
Polygon -7500403 true true 73 210 86 251 62 249 48 208
Polygon -7500403 true true 25 114 16 195 9 204 23 213 25 200 39 123

cross
false
14
Rectangle -16777216 true true 135 15 165 270
Rectangle -16777216 true true 75 105 195 105
Rectangle -16777216 true true 75 105 225 135
Rectangle -16777216 true true 120 45 180 60

cylinder
false
0
Circle -7500403 true true 0 0 300

dot
false
0
Circle -7500403 true true 90 90 120

face happy
false
0
Circle -7500403 true true 8 8 285
Circle -16777216 true false 60 75 60
Circle -16777216 true false 180 75 60
Polygon -16777216 true false 150 255 90 239 62 213 47 191 67 179 90 203 109 218 150 225 192 218 210 203 227 181 251 194 236 217 212 240

face neutral
false
0
Circle -7500403 true true 8 7 285
Circle -16777216 true false 60 75 60
Circle -16777216 true false 180 75 60
Rectangle -16777216 true false 60 195 240 225

face sad
false
0
Circle -7500403 true true 8 8 285
Circle -16777216 true false 60 75 60
Circle -16777216 true false 180 75 60
Polygon -16777216 true false 150 168 90 184 62 210 47 232 67 244 90 220 109 205 150 198 192 205 210 220 227 242 251 229 236 206 212 183

fish
false
0
Polygon -1 true false 44 131 21 87 15 86 0 120 15 150 0 180 13 214 20 212 45 166
Polygon -1 true false 135 195 119 235 95 218 76 210 46 204 60 165
Polygon -1 true false 75 45 83 77 71 103 86 114 166 78 135 60
Polygon -7500403 true true 30 136 151 77 226 81 280 119 292 146 292 160 287 170 270 195 195 210 151 212 30 166
Circle -16777216 true false 215 106 30

flag
false
0
Rectangle -7500403 true true 60 15 75 300
Polygon -7500403 true true 90 150 270 90 90 30
Line -7500403 true 75 135 90 135
Line -7500403 true 75 45 90 45

flower
false
0
Polygon -10899396 true false 135 120 165 165 180 210 180 240 150 300 165 300 195 240 195 195 165 135
Circle -7500403 true true 85 132 38
Circle -7500403 true true 130 147 38
Circle -7500403 true true 192 85 38
Circle -7500403 true true 85 40 38
Circle -7500403 true true 177 40 38
Circle -7500403 true true 177 132 38
Circle -7500403 true true 70 85 38
Circle -7500403 true true 130 25 38
Circle -7500403 true true 96 51 108
Circle -16777216 true false 113 68 74
Polygon -10899396 true false 189 233 219 188 249 173 279 188 234 218
Polygon -10899396 true false 180 255 150 210 105 210 75 240 135 240

hospital
false
0
Rectangle -1 true false 0 0 300 300
Rectangle -2674135 true false 0 105 300 195
Rectangle -2674135 true false 105 0 195 300

house
false
0
Rectangle -7500403 true true 45 120 255 285
Rectangle -16777216 true false 120 210 180 285
Polygon -7500403 true true 15 120 150 15 285 120
Line -16777216 false 30 120 270 120

leaf
false
0
Polygon -7500403 true true 150 210 135 195 120 210 60 210 30 195 60 180 60 165 15 135 30 120 15 105 40 104 45 90 60 90 90 105 105 120 120 120 105 60 120 60 135 30 150 15 165 30 180 60 195 60 180 120 195 120 210 105 240 90 255 90 263 104 285 105 270 120 285 135 240 165 240 180 270 195 240 210 180 210 165 195
Polygon -7500403 true true 135 195 135 240 120 255 105 255 105 285 135 285 165 240 165 195

line
true
0
Line -7500403 true 150 0 150 300

line half
true
0
Line -7500403 true 150 0 150 150

pentagon
false
0
Polygon -7500403 true true 150 15 15 120 60 285 240 285 285 120

person
false
0
Circle -7500403 true true 110 5 80
Polygon -7500403 true true 105 90 120 195 90 285 105 300 135 300 150 225 165 300 195 300 210 285 180 195 195 90
Rectangle -7500403 true true 127 79 172 94
Polygon -7500403 true true 195 90 240 150 225 180 165 105
Polygon -7500403 true true 105 90 60 150 75 180 135 105

plant
false
0
Rectangle -7500403 true true 135 90 165 300
Polygon -7500403 true true 135 255 90 210 45 195 75 255 135 285
Polygon -7500403 true true 165 255 210 210 255 195 225 255 165 285
Polygon -7500403 true true 135 180 90 135 45 120 75 180 135 210
Polygon -7500403 true true 165 180 165 210 225 180 255 120 210 135
Polygon -7500403 true true 135 105 90 60 45 45 75 105 135 135
Polygon -7500403 true true 165 105 165 135 225 105 255 45 210 60
Polygon -7500403 true true 135 90 120 45 150 15 180 45 165 90

sheep
false
15
Circle -1 true true 203 65 88
Circle -1 true true 70 65 162
Circle -1 true true 150 105 120
Polygon -7500403 true false 218 120 240 165 255 165 278 120
Circle -7500403 true false 214 72 67
Rectangle -1 true true 164 223 179 298
Polygon -1 true true 45 285 30 285 30 240 15 195 45 210
Circle -1 true true 3 83 150
Rectangle -1 true true 65 221 80 296
Polygon -1 true true 195 285 210 285 210 240 240 210 195 210
Polygon -7500403 true false 276 85 285 105 302 99 294 83
Polygon -7500403 true false 219 85 210 105 193 99 201 83

square
false
0
Rectangle -7500403 true true 30 30 270 270

square 2
false
0
Rectangle -7500403 true true 30 30 270 270
Rectangle -16777216 true false 60 60 240 240

star
false
0
Polygon -7500403 true true 151 1 185 108 298 108 207 175 242 282 151 216 59 282 94 175 3 108 116 108

target
false
0
Circle -7500403 true true 0 0 300
Circle -16777216 true false 30 30 240
Circle -7500403 true true 60 60 180
Circle -16777216 true false 90 90 120
Circle -7500403 true true 120 120 60

tree
false
0
Circle -7500403 true true 118 3 94
Rectangle -6459832 true false 120 195 180 300
Circle -7500403 true true 65 21 108
Circle -7500403 true true 116 41 127
Circle -7500403 true true 45 90 120
Circle -7500403 true true 104 74 152

triangle
false
0
Polygon -7500403 true true 150 30 15 255 285 255

triangle 2
false
0
Polygon -7500403 true true 150 30 15 255 285 255
Polygon -16777216 true false 151 99 225 223 75 224

truck
false
0
Rectangle -7500403 true true 4 45 195 187
Polygon -7500403 true true 296 193 296 150 259 134 244 104 208 104 207 194
Rectangle -1 true false 195 60 195 105
Polygon -16777216 true false 238 112 252 141 219 141 218 112
Circle -16777216 true false 234 174 42
Rectangle -7500403 true true 181 185 214 194
Circle -16777216 true false 144 174 42
Circle -16777216 true false 24 174 42
Circle -7500403 false true 24 174 42
Circle -7500403 false true 144 174 42
Circle -7500403 false true 234 174 42

turtle
true
0
Polygon -10899396 true false 215 204 240 233 246 254 228 266 215 252 193 210
Polygon -10899396 true false 195 90 225 75 245 75 260 89 269 108 261 124 240 105 225 105 210 105
Polygon -10899396 true false 105 90 75 75 55 75 40 89 31 108 39 124 60 105 75 105 90 105
Polygon -10899396 true false 132 85 134 64 107 51 108 17 150 2 192 18 192 52 169 65 172 87
Polygon -10899396 true false 85 204 60 233 54 254 72 266 85 252 107 210
Polygon -7500403 true true 119 75 179 75 209 101 224 135 220 225 175 261 128 261 81 224 74 135 88 99

wheel
false
0
Circle -7500403 true true 3 3 294
Circle -16777216 true false 30 30 240
Line -7500403 true 150 285 150 15
Line -7500403 true 15 150 285 150
Circle -7500403 true true 120 120 60
Line -7500403 true 216 40 79 269
Line -7500403 true 40 84 269 221
Line -7500403 true 40 216 269 79
Line -7500403 true 84 40 221 269

wolf
false
0
Polygon -16777216 true false 253 133 245 131 245 133
Polygon -7500403 true true 2 194 13 197 30 191 38 193 38 205 20 226 20 257 27 265 38 266 40 260 31 253 31 230 60 206 68 198 75 209 66 228 65 243 82 261 84 268 100 267 103 261 77 239 79 231 100 207 98 196 119 201 143 202 160 195 166 210 172 213 173 238 167 251 160 248 154 265 169 264 178 247 186 240 198 260 200 271 217 271 219 262 207 258 195 230 192 198 210 184 227 164 242 144 259 145 284 151 277 141 293 140 299 134 297 127 273 119 270 105
Polygon -7500403 true true -1 195 14 180 36 166 40 153 53 140 82 131 134 133 159 126 188 115 227 108 236 102 238 98 268 86 269 92 281 87 269 103 269 113

x
false
0
Polygon -7500403 true true 270 75 225 30 30 225 75 270
Polygon -7500403 true true 30 75 75 30 270 225 225 270
@#$#@#$#@
NetLogo 6.1.0
@#$#@#$#@
@#$#@#$#@
@#$#@#$#@
<experiments>
  <experiment name="exp03" repetitions="1" runMetricsEveryStep="false">
    <setup>setup</setup>
    <go>go</go>
    <exitCondition>end-of-run?</exitCondition>
    <metric>total-population</metric>
    <metric>ticks</metric>
    <metric>count people with [ state = "susceptible" ]</metric>
    <metric>count people with [ state = "infected" ]</metric>
    <metric>count people with [ state = "ill" ]</metric>
    <metric>count people with [ state = "recovered" ]</metric>
    <metric>count people with [ state = "immune" ]</metric>
    <metric>count people with [ state = "dead" ]</metric>
    <metric>count people with [ state = "infectious?" ]</metric>
    <metric>median [ conviviality * carelessness ] of people with [ state = "susceptible" ]</metric>
    <metric>median [ conviviality * carelessness ] of people with [ state != "susceptible" ]</metric>
    <metric>count people with [ wears-a-mask? ]</metric>
    <metric>count people with [ stays-at-home? ]</metric>
    <metric>first newly-infected-7-days-ma-max</metric>
    <metric>last newly-infected-7-days-ma-max</metric>
    <metric>first newly-deceased-7-days-ma-max</metric>
    <metric>last newly-deceased-7-days-ma-max</metric>
    <metric>no-more-infectious-at</metric>
    <metric>this-randomseed</metric>
    <metric>directory-name</metric>
    <metric>mean-distance-to-nearest-neighbour</metric>
    <enumeratedValueSet variable="normative-mode?">
      <value value="true"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="asymptomatic-per-infected">
      <value value="0.15"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="BACKGROUND">
      <value value="-2"/>
      <value value="0"/>
      <value value="2"/>
      <value value="4"/>
      <value value="6"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="case-fatality-rate">
      <value value="0.03"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="total-population">
      <value value="30000"/>
      <value value="70000"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="illness-duration-max">
      <value value="17"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="lockdown-end-at">
      <value value="25"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="distance-for-infection">
      <value value="2.25"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="immunity-duration-max">
      <value value="30"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="initial-counties">
      <value value="13"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="zero-person">
      <value value="&quot;densest&quot;"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="long-distance">
      <value value="50"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="immunity-duration-min">
      <value value="24"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="infectious-from-abroad">
      <value value="&quot;none&quot;"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="interruptible?">
      <value value="false"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="max-months">
      <value value="24"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="NDW">
      <value value="0.6"/>
      <value value="0.7"/>
      <value value="0.8"/>
      <value value="0.9"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="density-mode">
      <value value="&quot;d~1/area&quot;"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="short-distance">
      <value value="2"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="auto-lockdown?">
      <value value="false"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="far-commuter">
      <value value="0.2"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="reporting-rate">
      <value value="0.8"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="heterogeneity">
      <value value="&quot;both&quot;"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="time-till-immunity-min">
      <value value="0"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="infections-per-contact">
      <value value="0.25"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="time-to-infectiosity-min">
      <value value="1"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="lockdown-at">
      <value value="50"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="illness-duration-min">
      <value value="12"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="tested-per-infected">
      <value value="1"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="time-to-infectiosity-max">
      <value value="4"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="incubation-time-max">
      <value value="10"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="time-till-immunity-max">
      <value value="2"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="constant-seed?">
      <value value="true"/>
      <value value="false"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="incubation-time-min">
      <value value="5"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="reporting-delay">
      <value value="0"/>
    </enumeratedValueSet>
  </experiment>
  <experiment name="exp03-rest30k" repetitions="1" runMetricsEveryStep="false">
    <setup>setup</setup>
    <go>go</go>
    <exitCondition>end-of-run?</exitCondition>
    <metric>total-population</metric>
    <metric>ticks</metric>
    <metric>count people with [ state = "susceptible" ]</metric>
    <metric>count people with [ state = "infected" ]</metric>
    <metric>count people with [ state = "ill" ]</metric>
    <metric>count people with [ state = "recovered" ]</metric>
    <metric>count people with [ state = "immune" ]</metric>
    <metric>count people with [ state = "dead" ]</metric>
    <metric>count people with [ state = "infectious?" ]</metric>
    <metric>median [ conviviality * carelessness ] of people with [ state = "susceptible" ]</metric>
    <metric>median [ conviviality * carelessness ] of people with [ state != "susceptible" ]</metric>
    <metric>count people with [ wears-a-mask? ]</metric>
    <metric>count people with [ stays-at-home? ]</metric>
    <metric>first newly-infected-7-days-ma-max</metric>
    <metric>last newly-infected-7-days-ma-max</metric>
    <metric>first newly-deceased-7-days-ma-max</metric>
    <metric>last newly-deceased-7-days-ma-max</metric>
    <metric>no-more-infectious-at</metric>
    <metric>this-randomseed</metric>
    <metric>directory-name</metric>
    <metric>mean-distance-to-nearest-neighbour</metric>
    <enumeratedValueSet variable="normative-mode?">
      <value value="true"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="asymptomatic-per-infected">
      <value value="0.15"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="BACKGROUND">
      <value value="0"/>
      <value value="2"/>
      <value value="4"/>
      <value value="6"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="case-fatality-rate">
      <value value="0.03"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="total-population">
      <value value="30000"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="illness-duration-max">
      <value value="17"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="lockdown-end-at">
      <value value="25"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="distance-for-infection">
      <value value="2.25"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="immunity-duration-max">
      <value value="30"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="initial-counties">
      <value value="13"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="zero-person">
      <value value="&quot;densest&quot;"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="long-distance">
      <value value="50"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="immunity-duration-min">
      <value value="24"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="infectious-from-abroad">
      <value value="&quot;none&quot;"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="interruptible?">
      <value value="false"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="max-months">
      <value value="24"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="NDW">
      <value value="0.6"/>
      <value value="0.7"/>
      <value value="0.8"/>
      <value value="0.9"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="density-mode">
      <value value="&quot;d~1/area&quot;"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="short-distance">
      <value value="2"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="auto-lockdown?">
      <value value="false"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="far-commuter">
      <value value="0.2"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="reporting-rate">
      <value value="0.8"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="heterogeneity">
      <value value="&quot;both&quot;"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="time-till-immunity-min">
      <value value="0"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="infections-per-contact">
      <value value="0.25"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="time-to-infectiosity-min">
      <value value="1"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="lockdown-at">
      <value value="50"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="illness-duration-min">
      <value value="12"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="tested-per-infected">
      <value value="1"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="time-to-infectiosity-max">
      <value value="4"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="incubation-time-max">
      <value value="10"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="time-till-immunity-max">
      <value value="2"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="constant-seed?">
      <value value="true"/>
      <value value="false"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="incubation-time-min">
      <value value="5"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="reporting-delay">
      <value value="0"/>
    </enumeratedValueSet>
  </experiment>
  <experiment name="exp03-70k" repetitions="1" runMetricsEveryStep="false">
    <setup>setup</setup>
    <go>go</go>
    <exitCondition>end-of-run?</exitCondition>
    <metric>total-population</metric>
    <metric>ticks</metric>
    <metric>count people with [ state = "susceptible" ]</metric>
    <metric>count people with [ state = "infected" ]</metric>
    <metric>count people with [ state = "ill" ]</metric>
    <metric>count people with [ state = "recovered" ]</metric>
    <metric>count people with [ state = "immune" ]</metric>
    <metric>count people with [ state = "dead" ]</metric>
    <metric>count people with [ state = "infectious?" ]</metric>
    <metric>median [ conviviality * carelessness ] of people with [ state = "susceptible" ]</metric>
    <metric>median [ conviviality * carelessness ] of people with [ state != "susceptible" ]</metric>
    <metric>count people with [ wears-a-mask? ]</metric>
    <metric>count people with [ stays-at-home? ]</metric>
    <metric>first newly-infected-7-days-ma-max</metric>
    <metric>last newly-infected-7-days-ma-max</metric>
    <metric>first newly-deceased-7-days-ma-max</metric>
    <metric>last newly-deceased-7-days-ma-max</metric>
    <metric>no-more-infectious-at</metric>
    <metric>this-randomseed</metric>
    <metric>directory-name</metric>
    <metric>mean-distance-to-nearest-neighbour</metric>
    <enumeratedValueSet variable="normative-mode?">
      <value value="true"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="asymptomatic-per-infected">
      <value value="0.15"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="BACKGROUND">
      <value value="-2"/>
      <value value="0"/>
      <value value="2"/>
      <value value="4"/>
      <value value="6"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="case-fatality-rate">
      <value value="0.03"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="total-population">
      <value value="70000"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="illness-duration-max">
      <value value="17"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="lockdown-end-at">
      <value value="25"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="distance-for-infection">
      <value value="2.25"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="immunity-duration-max">
      <value value="30"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="initial-counties">
      <value value="13"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="zero-person">
      <value value="&quot;densest&quot;"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="long-distance">
      <value value="50"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="immunity-duration-min">
      <value value="24"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="infectious-from-abroad">
      <value value="&quot;none&quot;"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="interruptible?">
      <value value="false"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="max-months">
      <value value="24"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="NDW">
      <value value="0.6"/>
      <value value="0.7"/>
      <value value="0.8"/>
      <value value="0.9"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="density-mode">
      <value value="&quot;d~1/area&quot;"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="short-distance">
      <value value="2"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="auto-lockdown?">
      <value value="false"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="far-commuter">
      <value value="0.2"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="reporting-rate">
      <value value="0.8"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="heterogeneity">
      <value value="&quot;both&quot;"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="time-till-immunity-min">
      <value value="0"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="infections-per-contact">
      <value value="0.25"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="time-to-infectiosity-min">
      <value value="1"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="lockdown-at">
      <value value="50"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="illness-duration-min">
      <value value="12"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="tested-per-infected">
      <value value="1"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="time-to-infectiosity-max">
      <value value="4"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="incubation-time-max">
      <value value="10"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="time-till-immunity-max">
      <value value="2"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="constant-seed?">
      <value value="true"/>
      <value value="false"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="incubation-time-min">
      <value value="5"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="reporting-delay">
      <value value="0"/>
    </enumeratedValueSet>
  </experiment>
  <experiment name="exp03-70k32" repetitions="1" runMetricsEveryStep="false">
    <setup>setup</setup>
    <go>go</go>
    <exitCondition>end-of-run?</exitCondition>
    <metric>total-population</metric>
    <metric>ticks</metric>
    <metric>count people with [ state = "susceptible" ]</metric>
    <metric>count people with [ state = "infected" ]</metric>
    <metric>count people with [ state = "ill" ]</metric>
    <metric>count people with [ state = "recovered" ]</metric>
    <metric>count people with [ state = "immune" ]</metric>
    <metric>count people with [ state = "dead" ]</metric>
    <metric>count people with [ state = "infectious?" ]</metric>
    <metric>median [ conviviality * carelessness ] of people with [ state = "susceptible" ]</metric>
    <metric>median [ conviviality * carelessness ] of people with [ state != "susceptible" ]</metric>
    <metric>count people with [ wears-a-mask? ]</metric>
    <metric>count people with [ stays-at-home? ]</metric>
    <metric>first newly-infected-7-days-ma-max</metric>
    <metric>last newly-infected-7-days-ma-max</metric>
    <metric>first newly-deceased-7-days-ma-max</metric>
    <metric>last newly-deceased-7-days-ma-max</metric>
    <metric>no-more-infectious-at</metric>
    <metric>this-randomseed</metric>
    <metric>directory-name</metric>
    <metric>mean-distance-to-nearest-neighbour</metric>
    <enumeratedValueSet variable="normative-mode?">
      <value value="true"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="asymptomatic-per-infected">
      <value value="0.15"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="BACKGROUND">
      <value value="0"/>
      <value value="2"/>
      <value value="4"/>
      <value value="6"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="case-fatality-rate">
      <value value="0.03"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="total-population">
      <value value="70000"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="illness-duration-max">
      <value value="17"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="lockdown-end-at">
      <value value="25"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="distance-for-infection">
      <value value="2.25"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="immunity-duration-max">
      <value value="30"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="initial-counties">
      <value value="13"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="zero-person">
      <value value="&quot;densest&quot;"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="long-distance">
      <value value="50"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="immunity-duration-min">
      <value value="24"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="infectious-from-abroad">
      <value value="&quot;none&quot;"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="interruptible?">
      <value value="false"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="max-months">
      <value value="24"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="NDW">
      <value value="0.6"/>
      <value value="0.7"/>
      <value value="0.8"/>
      <value value="0.9"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="density-mode">
      <value value="&quot;d~1/area&quot;"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="short-distance">
      <value value="2"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="auto-lockdown?">
      <value value="false"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="far-commuter">
      <value value="0.2"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="reporting-rate">
      <value value="0.8"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="heterogeneity">
      <value value="&quot;both&quot;"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="time-till-immunity-min">
      <value value="0"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="infections-per-contact">
      <value value="0.25"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="time-to-infectiosity-min">
      <value value="1"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="lockdown-at">
      <value value="50"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="illness-duration-min">
      <value value="12"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="tested-per-infected">
      <value value="1"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="time-to-infectiosity-max">
      <value value="4"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="incubation-time-max">
      <value value="10"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="time-till-immunity-max">
      <value value="2"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="constant-seed?">
      <value value="true"/>
      <value value="false"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="incubation-time-min">
      <value value="5"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="reporting-delay">
      <value value="0"/>
    </enumeratedValueSet>
  </experiment>
  <experiment name="exp01a" repetitions="1" runMetricsEveryStep="false">
    <setup>setup</setup>
    <go>go</go>
    <exitCondition>end-of-run?</exitCondition>
    <metric>total-population</metric>
    <metric>ticks</metric>
    <metric>count people with [ state = "susceptible" ]</metric>
    <metric>count people with [ state = "infected" ]</metric>
    <metric>count people with [ state = "ill" ]</metric>
    <metric>count people with [ state = "recovered" ]</metric>
    <metric>count people with [ state = "immune" ]</metric>
    <metric>count people with [ state = "dead" ]</metric>
    <metric>count people with [ state = "infectious?" ]</metric>
    <metric>median [ conviviality * carelessness ] of people with [ state = "susceptible" ]</metric>
    <metric>median [ conviviality * carelessness ] of people with [ state != "susceptible" ]</metric>
    <metric>count people with [ wears-a-mask? ]</metric>
    <metric>count people with [ stays-at-home? ]</metric>
    <metric>first newly-infected-7-days-ma-max</metric>
    <metric>last newly-infected-7-days-ma-max</metric>
    <metric>first newly-deceased-7-days-ma-max</metric>
    <metric>last newly-deceased-7-days-ma-max</metric>
    <metric>no-more-infectious-at</metric>
    <metric>this-randomseed</metric>
    <metric>directory-name</metric>
    <metric>mean-distance-to-nearest-neighbour</metric>
    <enumeratedValueSet variable="normative-mode?">
      <value value="true"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="asymptomatic-per-infected">
      <value value="0.15"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="BACKGROUND">
      <value value="-2"/>
      <value value="0"/>
      <value value="2"/>
      <value value="4"/>
      <value value="6"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="case-fatality-rate">
      <value value="0.03"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="total-population">
      <value value="10000"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="illness-duration-max">
      <value value="17"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="lockdown-end-at">
      <value value="25"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="distance-for-infection">
      <value value="2.25"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="immunity-duration-max">
      <value value="30"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="initial-counties">
      <value value="13"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="zero-person">
      <value value="&quot;densest&quot;"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="long-distance">
      <value value="50"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="immunity-duration-min">
      <value value="24"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="infectious-from-abroad">
      <value value="&quot;none&quot;"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="interruptible?">
      <value value="false"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="max-months">
      <value value="24"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="NDW">
      <value value="0.6"/>
      <value value="0.7"/>
      <value value="0.8"/>
      <value value="0.9"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="density-mode">
      <value value="&quot;d~1/area&quot;"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="short-distance">
      <value value="2"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="auto-lockdown?">
      <value value="false"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="far-commuter">
      <value value="0.2"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="reporting-rate">
      <value value="0.8"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="heterogeneity">
      <value value="&quot;both&quot;"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="time-till-immunity-min">
      <value value="0"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="infections-per-contact">
      <value value="0.25"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="time-to-infectiosity-min">
      <value value="1"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="lockdown-at">
      <value value="50"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="illness-duration-min">
      <value value="12"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="tested-per-infected">
      <value value="1"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="time-to-infectiosity-max">
      <value value="4"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="incubation-time-max">
      <value value="10"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="time-till-immunity-max">
      <value value="2"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="constant-seed?">
      <value value="true"/>
      <value value="false"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="incubation-time-min">
      <value value="5"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="reporting-delay">
      <value value="0"/>
    </enumeratedValueSet>
  </experiment>
  <experiment name="test3" repetitions="1" runMetricsEveryStep="false">
    <setup>setup</setup>
    <go>go</go>
    <exitCondition>end-of-run?</exitCondition>
    <metric>total-population</metric>
    <metric>ticks</metric>
    <metric>count people with [ state = "susceptible" ]</metric>
    <metric>count people with [ state = "infected" ]</metric>
    <metric>count people with [ state = "ill" ]</metric>
    <metric>count people with [ state = "recovered" ]</metric>
    <metric>count people with [ state = "immune" ]</metric>
    <metric>count people with [ state = "dead" ]</metric>
    <metric>count people with [ state = "infectious?" ]</metric>
    <metric>median [ conviviality * carelessness ] of people with [ state = "susceptible" ]</metric>
    <metric>median [ conviviality * carelessness ] of people with [ state != "susceptible" ]</metric>
    <metric>count people with [ wears-a-mask? ]</metric>
    <metric>count people with [ stays-at-home? ]</metric>
    <metric>first newly-infected-7-days-ma-max</metric>
    <metric>last newly-infected-7-days-ma-max</metric>
    <metric>first newly-deceased-7-days-ma-max</metric>
    <metric>last newly-deceased-7-days-ma-max</metric>
    <metric>no-more-infectious-at</metric>
    <metric>this-randomseed</metric>
    <metric>directory-name</metric>
    <metric>mean-distance-to-nearest-neighbour</metric>
    <enumeratedValueSet variable="normative-mode?">
      <value value="true"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="asymptomatic-per-infected">
      <value value="0.15"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="BACKGROUND">
      <value value="-2"/>
      <value value="0"/>
      <value value="2"/>
      <value value="4"/>
      <value value="6"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="case-fatality-rate">
      <value value="0.03"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="total-population">
      <value value="10000"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="illness-duration-max">
      <value value="17"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="lockdown-end-at">
      <value value="25"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="distance-for-infection">
      <value value="2.25"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="immunity-duration-max">
      <value value="30"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="initial-counties">
      <value value="13"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="zero-person">
      <value value="&quot;densest&quot;"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="long-distance">
      <value value="50"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="immunity-duration-min">
      <value value="24"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="infectious-from-abroad">
      <value value="&quot;none&quot;"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="interruptible?">
      <value value="false"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="max-months">
      <value value="24"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="NDW">
      <value value="0.8"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="density-mode">
      <value value="&quot;d~1/area&quot;"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="short-distance">
      <value value="2"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="auto-lockdown?">
      <value value="false"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="far-commuter">
      <value value="0.2"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="reporting-rate">
      <value value="0.8"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="heterogeneity">
      <value value="&quot;both&quot;"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="time-till-immunity-min">
      <value value="0"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="infections-per-contact">
      <value value="0.25"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="time-to-infectiosity-min">
      <value value="1"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="lockdown-at">
      <value value="50"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="illness-duration-min">
      <value value="12"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="tested-per-infected">
      <value value="1"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="time-to-infectiosity-max">
      <value value="4"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="incubation-time-max">
      <value value="10"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="time-till-immunity-max">
      <value value="2"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="constant-seed?">
      <value value="true"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="incubation-time-min">
      <value value="5"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="reporting-delay">
      <value value="0"/>
    </enumeratedValueSet>
  </experiment>
</experiments>
@#$#@#$#@
@#$#@#$#@
default
0.0
-0.2 0 0.0 1.0
0.0 1 1.0 0.0
0.2 0 0.0 1.0
link direction
true
0
Line -7500403 true 150 150 90 180
Line -7500403 true 150 150 210 180
@#$#@#$#@
0
@#$#@#$#@
